function imgNoisy = add(img, varargin)
%TODO: change inputs from gaussianStd, maxPxlSensor to gaussianStdSensor, scaler) where scaler defines current pixel /
%sensorPixel
%Add Gaussian or/and Possion Noise to a 2D/3D/nD image
%imgNoisy = noise.add(img, 'gaussianStd', gaussianStd, 'maxPxlSensor', maxPxlSensor, 'maxPhoton', maxPhoton, 'rngState', 'default')
% Input image must be uint8/uint16/single/double class
% Output image will be single/single/single/double class correspondingly (in future may change to keep uint8/uint16 only if neccessary)

%
% Inputs:
%   img:        Input 2D/3D/nD image array
%   gaussianStd: std of Gaussian noise to be added, in the sensor pixel units, NOT in the current pixel units
%               However, if the img is the direct read out from the sensor without any scaling, or it is the direct read out
%               subtract a background, then the img is not scaled, we should havemaxPxlSensor == max(img(:)).
%   maxPxlSensor:   Opitional argument, the pixel counts for the brightest pixel in the sensor pixel units.
%               If it is not specified, we assume current pixel count is the actual pixel count
%
% Outputs:
%   imgNoisy:   Input image with Poisson noise added.
%               It has the same dimension as img.
%               It is the same class as img, except when img is uint8/uint16 but maxPxlSensor is not equal to max(img(:))
%               then it will be single float
% Usage:
%   imgNoisy = noise.addGaussian(img, 38); % 38 is typical MFM sensor std
%   imgNoisy = noise.addGaussian(img, 38, 60000); % 60000 is the brightest pixel in original sensor pixel unit
%
%   If we were given the pixelMean instead of maxPxlSensor, we should compute maxPxlSensor before call the function
%   maxPxlSensor = single(pixelMean) / mean(single(img(:))) * single(max(img(:));
%   Similarly if we were given the pixelSum,
%   maxPxlSensor = single(pixelSum) / sum(single(img(:))) * single(max(img(:));

%
% 2017.04.23 created
% Author: Xiang Huang: xianghuang@gmail.com
%
% NOTEs:
%   1. Do we threshold negative pixels to 0 in imgNoisy? Currently No.


isNoneNegativeNumber	= @(x) isnumeric(x) && isscalar(x) && (x>=0);
%isPositiveInteger   = @(x) isnumeric(x) && isscalar(x) && (x>0) && (x==round(x));
p = inputParser;  % CaseSensitive=0, PartialMatching=1
p.addParameter('maxPhoton',     inf,    isNoneNegativeNumber);
p.addParameter('gaussianStd',   0,      isNoneNegativeNumber);
p.addParameter('maxPxlSensor',  0,      isNoneNegativeNumber);  % change to is isNoneNegativeInt?
p.addParameter('rngState',      'shuffle',  @(x) ischar(x) && ismember(lower(x), {'shuffle', 'default'})); % by default generate different random numbers each time.
p.parse(varargin{:});
cellfun(@(x) evalin('caller', sprintf('%s=p.Results.%s;', x, x)), fields(p.Results)); % e.g. eval('fSz = p.Results.fSz'); Or use in.fSz after set in =p.Results;

rng(rngState);
imgNoisyPoisson = noise.addPoisson(img, maxPhoton); % the std for brightest pixel is around 2000 = gain*sqrt(maxPhoton)
rng(rngState);
imgNoisy =  imgNoisyPoisson + (noise.addGaussian(img, gaussianStd, maxPxlSensor) - img); % the std for brightest pixel is around 2000 = gain*sqrt(maxPhoton)

end

%% Unit Test
function testMe()

end