function [rectPos, corn] = calcLargestInscribedImage(im, isShow)
%calcLargestInscribedImage
% 2017.12.14
% Output:   rectPos = [xMin, yMin, width, height]
%           four corners
% get from
% https://stackoverflow.com/questions/30133895/how-can-i-detect-the-maximum-sized-rectangle-that-i-can-draw-onto-the-mask
% ----------------------------------------------------------
% Here is a correct answer. You must use dynamic programming! Other methods of direct calculation (like cutting 1 pixel from each edge) might produce sub-optimal results. My method guarantees that it selects the largest possible rectangle that fits in the mask. I assume that the mask has 1 big convex white blob of any shape with black background around it.
% 
% I wrote 2 methods. findRect() which finds the best possible square (starting on x,y with length l). The second method LargestInscribedImage() is an example of how to find any rectangle (of any aspect ratio). The trick is to resize the mask image, find a square and resize it back. In my example the method finds the larges rectangle that can be fit in the mask having the same aspect ration as the mask image. For example if the mask image is of size 100x200 pixels than the algorithm will find the largest rectangle having aspect ratio 1:2.
% 	
% Please read in wikipedia about dynamic programing. The trick is to solve the problem to a smaller image first. Say if you have an image NxN and someone gives you a solution for sub image (N-1)x(N-1) You can easily add the last line and row to the solution, and get the full solution. So I start with a tiny image (1xN and Nx1) and solve in the initialization step. The next step in the for loop assumes existing solution for MxM image and extends it to (M+1)xM, then to Mx(M+1) and finally to MxM. – DanielHsH May 9 '15 at 14:58
%    	 	
% Please note: My algorithm will not work if your mask has holes (like white are of a donuts shape) The mask must have 1 big blob without holes. It basically checks all possible solutions but does it in N^2 steps instead of Naive N^8 steps implementation (checking all possible combinations of 4 corners). That is why this algorithm guarantees the largest square for a convex shape of the blob. It will work pretty well for non convex shapes as well, but can give sub-optimal results in extremely difficult cases. – DanielHsH May 9 '15 at 15:02 

% Balck and white image of your mask


if ~exist('isShow', 'var') || isempty(isShow)
    isShow = false;
end

%%
b = imresize(im,[size(im,1) size(im,1)]); % Make the mask square by resizing it by its aspect ratio.

SC = 1;  % Put 2..4 to scale down the image an speed up the algorithm

[x1,y1,l1] = findRect(b,SC);          % Lunch the dyn prog algorithm
[x2,y2,l2] = findRect(rot90(b),SC);   % rotate the image by 90deg and solve
% Rotate back: x2,y2 according to rot90
tmp = x2;
x2 = size(im,1)/SC-y2-l2;
y2 = tmp;

% Select the best solution of the above (for the original image and for the rotated by 90degrees
if (l1>=l2)
    corn = sqCorn(x1,y1,l1);
else
    corn = sqCorn(x2,y2,l2);
end

b = imresize(b,1/SC);
rectPos = round([corn(1,1), corn(2,1), corn(1,end)-corn(1,1), corn(2,end)-corn(2,1)]); % xMin, yMin, width, height
if isShow
    figure; subplot(121); imshow(b); hold on;
    rectangle('position', rectPos, 'edgeColor', 'r')
end

corn = corn*SC;
corn(1,:) = corn(1,:)*size(im, 2) / size(im,1);
rectPos = round([corn(1,1), corn(2,1), corn(1,end)-corn(1,1), corn(2,end)-corn(2,1)]); % xMin, yMin, width, height
if isShow
    subplot(122);imshow(im); hold on;    
    rectangle('position', rectPos, 'edgeColor', 'r')
end

end

function corn = sqCorn(x,y,l)
corn = [x,y;x,y+l;x+l,y;x+l,y+l]';
end

% ----------------------------------------------------------
function [x,y,l] = findRect(b,SC)
b = imresize(b,1/SC);
res = zeros(size(b,1),size(b,2),3);
% initialize first col
for i = 1:1:size(b,1)
    if (b(i,1) > 0)
        res(i,1,:) = [i,1,0];
    end
end
% initialize first row
for i = 1:1:size(b,2)
    if (b(1,i) > 0)
        res(1,i,:) = [1,i,0];
    end
end

% DynProg
for i = 2:1:size(b,1)
    for j = 2:1:size(b,2)
        isWhite = b(i,j) > 0;
        if (~isWhite)
            res(i,j,:)=res(i-1,j-1,:); % copy
        else
            if (b(i-1,j-1)>0)  % continuous line
                lineBeg    = [res(i-1,j-1,1),res(i-1,j-1,2)];
                lineLenght = res(i-1,j-1,3);
                if ((b(lineBeg(1),j)>0)&&(b(i,lineBeg(2))>0))  % if second diag is good
                    res(i,j,:) = [lineBeg,lineLenght+1];
                else
                    res(i,j,:)=res(i-1,j-1,:); % copy since line has ended
                end
            else
                res(i,j,:) = [i,j,0];         % Line start
            end
        end
        
    end
end

% check last col
[maxValCol,WhereCol] = max(res(:,end,3));
% check last row
[maxValRow,WhereRow] = max(res(end,:,3));

% Find max
x= 0; y = 0; l = 0;
if (maxValCol>maxValRow)
    y = res(WhereCol,end,1);
    x = res(WhereCol,end,2);
    l = maxValCol;
else
    y = res(end,WhereRow,1);
    x = res(end,WhereRow,2);
    l = maxValRow;
end

corn = [x,y;x,y+l;x+l,y;x+l,y+l]';
%    figure;imshow(b>0); hold on;
%   plot(corn(1,:),corn(2,:),'O')
end

