readme = 'cleanBackgroundUseTiles_Script and cleanBackgroundUseTiles: Masked by tile, Removed Non Uniform Cam Offset, normalized by gain, then each tile subtract constant background from mean of furthest 1000 pixels to center';

folder = '/sandbox/smallWorlds/20171204_Insulin_PSF_200nm/';

load(fullfile(folder, 'tiles.mat')); 

% load structure tiles with fields:
%       maskEdgeFlooded: [1024×1024 logical]
%           maskFlooded: [1024×1024 logical]
%           centroidXYs: [2×9 double]
%     outerRectPosXYWHs: [4×9 double]
%     innerRectPosXYWHs: [4×9 double]
%         maskOuterRect: [1024×1024 logical]
%         maskInnerRect: [1024×1024 logical]
%     maskEdgeOuterRect: [1024×1024 logical]
%     maskEdgeInnerRect: [1024×1024 logical]

%% psf
psfOrigUint16 = tif.read(fullfile(folder, 'PSF_200nm_ex900em30_f241.tif'));

%%
Nbg = 50000; % try 1000 & 10000 & 50000 for each tile (can't be more than 1024^2/NTile = 1.1e5) 
tag = sprintf('PSF_200nm_ex900em30_f241_Nbg%d', Nbg);
pages = (1:241)';
cam = CAMERA('EMGain', 30, 'camOffsetFile', '/sandbox/Dropbox/smallWorlds/20170314_Camera/camOffset.tif');
[psf, psfTilePhoton] = preprocess.cleanBackgroundUseTiles(psfOrigUint16, tiles, cam, pages, folder, tag, Nbg);
save(fullfile(folder, [tag '_bgRemoved.mat']), 'readme', 'psf', 'psfTilePhoton', 'tiles', 'cam', 'pages', 'Nbg');

preprocess.showImgTilePhoton(psfTilePhoton); printfig(fullfile(folder, [tag '_bgRemoved_Analysis.png']));

psfUint16 = psf * (65535/max(psf(:))); 
psfUint16(psfUint16<0) = 0;
psfUint16 = cast(round(psfUint16), 'uint16');
tif.write(psfUint16, fullfile(folder, sprintf('%s_bgRemoved_maxPhoton%d.tif', tag, round(max(psf(:))))));

%% Insulin
Nbg = 10000;  % try 1000 & 10000 & 50000 for each tile (can't be more than 1024^2/NTile = 1.1e5) 
insulinFileAll = {
    fullfile(folder, 'insulin_ex98em750_f1000.tif');    
    };
tagAll = {
    sprintf('insulin_ex98em750_f1000_Nbg%d', Nbg);
         };
cam = CAMERA('EMGain', 750, 'camOffsetFile', '/sandbox/Dropbox/smallWorlds/20170314_Camera/camOffset.tif');
pages = [];

for nFile = 1:numel(insulinFileAll)    
    gOrigUint16 = tif.read(insulinFileAll{nFile});
    tag = tagAll{nFile};
    [g, gTilePhoton, gPlusBg] = preprocess.cleanBackgroundUseTiles(gOrigUint16, tiles, cam, pages, folder, tag, Nbg);
    %Warning: Variable 'g' was not saved. For variables larger than 2GB use MAT-file version 7.3 or later. 
    
    save(fullfile(folder, [tag '_bgRemoved.mat']), 'readme', 'g', 'gTilePhoton', 'tiles', 'cam', 'pages', 'Nbg', '-v7.3');
    
    preprocess.showImgTilePhoton(gTilePhoton); printfig(fullfile(folder, [tag '_bgRemoved_Analysis.png']));    
    
    gUint16 = g * (65535/max(g(:)));
    gUint16(gUint16<0) = 0;
    gUint16 = cast(round(gUint16), 'uint16');
    tif.write(gUint16, fullfile(folder, sprintf('%s_bgRemoved_maxPhoton%d.tif', tag, round(max(g(:))))));
end
