% 2018.05.10 Convert tif image from uint16 digital unit (raw image) to float32 photon image. 
% We subtract the constant camera offset (though x,y spatially variant), and divide by the gain, to convert from digital unit (DU) to photon
% We will/may? also apply filed-stop mask if there is any, so that all pixels outside the mask will be set to zero.
% Above photon counts include the backtround photon and psf/bacteria photon. 
%
%% 20180516 data
folder = '/sandbox/smallWorlds/20180516_Peri_PSF_MFM3x3_Confocal';
camAll = {CAMERA('EMGain', 100, 'camOffsetFile', '/sandbox/Dropbox/smallWorlds/20170314_Camera/camOffset.tif');
        CAMERA('EMGain', 750, 'camOffsetFile', '/sandbox/Dropbox/smallWorlds/20170314_Camera/camOffset.tif');};
tifFileNameAllAll = {{'peri_1_confocal_ex100em100_dz50nm_f162', 'PSF_confocal_fb170nm_ex100em100_dz50nm_f181', 'PSF_f201'};
                   {'peri_1'}};
pages = [];

for nCam  = 1:numel(camAll)
    cam = camAll{nCam};
    tifSuffix = sprintf('_PhU_GsStd%.2f', cam.gaussianStdDU/cam.gain);
    tifFileNameAll = tifFileNameAllAll{nCam};
    for nFile = 1:numel(tifFileNameAll)
        tifFileName = tifFileNameAll{nFile};
        convertDU2PhotonFun(folder, tifFileName, pages, tifSuffix, cam)
    end
end


%% 20180507 data
folder = '/sandbox/Dropbox/smallWorlds/20180507_PSF_100nm_MFM3x3';
camPSF = CAMERA('EMGain', 100, 'camOffsetFile', '/sandbox/Dropbox/smallWorlds/20170314_Camera/camOffset.tif');
tifFileNameAll = {'PSF_100nm_ex900em100_f201'};
pages = [];

cam = camPSF;
tifSuffix = sprintf('_PhU_GsStd%.2f', cam.gaussianStdDU/cam.gain);
for nFile = 1:numel(tifFileNameAll)
    tifFileName = tifFileNameAll{nFile};
    convertDU2PhotonFun(folder, tifFileName, pages, tifSuffix, cam)
end

%% 20160809 data
folder = '/sandbox/smallWorlds/20160809_Bacterial_Confocal_PSFs';
%% PSF
camPSF = CAMERA('EMGain', 750, 'camOffsetFile', '/sandbox/Dropbox/smallWorlds/20170314_Camera/camOffset.tif');
tifFileNameAll = {'PSF_1', 'PSF_2', 'PSF_3', 'PSF_4', 'PSF_Ave'};
pages = 151:251;
tifSuffix = '_PhotonUnit_Page151to251';

cam = camPSF;
for nFile = 1:numel(tifFileNameAll)
    tifFileName = tifFileNameAll{nFile};
    convertDU2PhotonFun(folder, tifFileName, pages, tifSuffix, cam)
end

%% Bacteria
camBac = CAMERA('EMGain', 300, 'camOffsetFile', '/sandbox/Dropbox/smallWorlds/20170314_Camera/camOffset.tif');
tifFileNameAll = {'Bacteria1_DoubleLobes_MFM_10Measures', 'Bacteria2_LongZ_MFM_10Measures'};
pages = [];
tifSuffix = '_PhotonUnit';

cam = camBac;
for nFile = 1:numel(tifFileNameAll)
    tifFileName = tifFileNameAll{nFile};
    convertDU2PhotonFun(folder, tifFileName, pages, tifSuffix, cam)
end

%%

%%
function convertDU2PhotonFun(folder, tifFileName, pages, tifSuffix, cam)        
    tifFile = fullfile(folder, [tifFileName, '.tif']);
    imgAllUint16 = tif.read(tifFile, pages);    
    fprintf('Convert tifFile %s (%d pages) from digital units to photon units with settings EMgain=%d, gain=%.2f.\n', tifFileName,size(imgAllUint16,3), cam.EMGain, cam.gain);     
    
    imgAllPhotonUnit = bsxfun(@minus, single(imgAllUint16), single(cam.camOffSet));
    imgAllPhotonUnit = imgAllPhotonUnit / single(cam.gain);
    tifPhotonFile = fullfile(folder, [tifFileName, tifSuffix '.tif']);
    tif.write(imgAllPhotonUnit, tifPhotonFile);
end