files = dir('Mask*.bmp');
N = numel(files);
Ny = 1024; Nx = 1024;
gap = 5;
pages = 151:gap:251;
assert(N==numel(pages), 'Inconsistent total number of pages');
%% convert from uint8 (orignal bmp from imagej is uint8) to logical
%for n = 1:N; file = files(n).name; I = imread(file); I=I>0; imwrite(I, file); end

%% Generate tif for mask with gap
IAllGappedBinary = false(Ny, Nx, N);
for n = 1:N
    file = files(n).name; 
    I = imread(file); 
    IAllGappedBinary(:,:,n) =  I > 0 ;  %image j only views uint8 tif, can't view binary tif.
end
IAllGapped = zeros(size(IAllGappedBinary), 'uint8'); IAllGapped(IAllGappedBinary>0) = 255;
tif.write(IAllGapped, 'PSF_Ave_Mask3D_151to251_gap5.tif');

%% Generate tif for mask without gap, interpolation
%% 1 compute the radius and centers for each circle in each frame
% find circles, just use connected component analysis,  no need to use imfindcircles()
NTile = 25; 
y0 = zeros(N, NTile);
x0 = zeros(N, NTile);
radius = zeros(N, NTile);
for n = 1:N
    I = IAllGapped(:,:,n)>0; 
    CC = bwconncomp(I);
    assert(CC.NumObjects == NTile, sprintf('The number of connected components in mask %d does not match the number of tiles %d', CC.NumObjects, NTile));
    for t = 1:NTile
        pxlIdxs = CC.PixelIdxList{t};
        radius(n,t) = sqrt(numel(pxlIdxs)/pi);
        [y, x] = ind2sub([Ny, Nx], pxlIdxs);
        y0(n, t) = mean(y);
        x0(n, t) = mean(x);        
    end
end

%% 2. make sure each cirle is in correct tile
% use the first frame as baseline, 
idx = zeros(N, NTile);
for n = 1:N %2:N     
    for t = 1:NTile
        distCenters = (y0(1,t)-y0(n,:)).^2  + (x0(1,t)-x0(n,:)).^2;
        [~, idx(n, t)] = min(distCenters);        
    end    
    tmp = y0(n,:); y0(n, :) = tmp(idx(n,:));
    tmp = x0(n,:); x0(n, :) = tmp(idx(n,:));
    tmp = radius(n,:); radius(n, :) = tmp(idx(n,:));
end


%% 3. Interpolate from gapped N frames to no-gap N2 frames
pages = pages - pages(1) + 1; % gapped pages: shift to make the first page as index 1, corresponding to IAllGapped(:,:,1) with circle center at y0(1,:), x0(1,:), and radius(1, :)
N2 = pages(end)-pages(1) +1;
pages2 = 1:N2; % none gapped pages;
% for each frame i in pages2, find index L (left) and R (right) in pages, so that pages(L) < page <pages(R), or L=R and pages(L)=page=pages(R)
% L is the index so that pages(L) is the largest frame in pages that is smaller than page
% R is the index so that pages(R) is the smallest frame in pages that is larger than page
% hack, pages has constant gap for now
L = floor((pages2 -1)/gap)+1;
R = ceil((pages2 -1)/gap)+1;
% compute the weight of contribution from pages(L) and pages(R)
wL =  (pages(R)- pages2) ./ (pages(R) - pages(L));
wR =  (pages2- pages(L)) ./ (pages(R) - pages(L));
% handle cases when L=R
idx = (pages(R) - pages(L)) ==0;
wL(idx) = 1;
wR(idx) = 0;
radiusInterp = zeros(N2, NTile);
y0Interp = zeros(N2, NTile);
x0Interp = zeros(N2, NTile);
for t = 1:NTile    
    radiusInterp(:,t) = wL' .* radius(L,t)  +  wR' .* radius(R,t);
    y0Interp(:,t) = wL' .* y0(L,t)  +  wR' .* y0(R,t);
    x0Interp(:,t) = wL' .* x0(L,t)  +  wR' .* x0(R,t);
end
% 
IAll = false(Ny, Nx, N2);
for n = 1:N2
    I = false(Ny, Nx);
    for t = 1:NTile
        yxInCircle = shape.getCoordsInCircle(radiusInterp(n,t), [y0Interp(n,t), x0Interp(n,t)]);
        pxlIdxInCircle = sub2ind([Ny, Nx], yxInCircle(:,1), yxInCircle(:,2));        
        I(pxlIdxInCircle) = true;
    end
    IAll(:, :, n) = I;
end
%%
IAllUint8 = zeros(size(IAll), 'uint8'); IAllUint8(IAll>0) = 255;
tif.write(IAllUint8, 'PSF_Ave_Mask3D_151to251.tif');
%% faked mask for all 401 pages for page 1:150 and 252:401
IAll401Pages = zeros(Ny, Nx, 401, 'uint8');
IAll401Pages(:,:,151:251) = IAllUint8;
IAll401Pages(:,:,1:150) = repmat(IAllUint8(:,:,1), [1,1,150]);
IAll401Pages(:,:,252:end) = repmat(IAllUint8(:,:, end), [1,1,150]);
tif.write(IAll401Pages, 'PSF_Ave_Mask3D.tif');

%%
% CC2 = bwconncomp(I2);
% CC = CC1;
% for t = 6%:CC.NumObjects    
% end
% %
% INew = false(Ny, Nx);
% INew(pxlIdxs) = 1;
% figure(1), imshow(INew);
% 
% INew2 = false(Ny, Nx);
% yxInCircle = shape.getCoordsInCircle(radius, [y0, x0]);
% pxlIdxInCircle = sub2ind([Ny, Nx], yxInCircle(:,1), yxInCircle(:,2));
% INew2(pxlIdxInCircle) = 1;
% figure(2), imshow(INew2);
