%2018.05.20, remove constant bg for each tile, 
% assume tiles are aligned blocks, compute bg as the mean of pixels at the border of each tile.
% refer to cleanBackground

%% 20180516 data
folder = '/sandbox/smallWorlds/20180516_Peri_PSF_MFM3x3_Confocal';
tifFileNameAll = {'PSF_PhU_GsStd1.84_fs_rvTileConstBg.tif'};%, ...                    
% same as start from 'PSF_PhU_GsStd1.84_fs_rvTileConstBg.tif';
%                     'peri_1_PhU_GsStd0.27_wSum_fs', ...
%                     'floodedIris_MFM_f100_fs', ...
%                     'floodedIris_MFM_f100_sum_fs'};
maskFileNameAll = {'PSF_Mask2D.bmp'};
pages = [];

tifSuffix = '_rvMaskedBg';


%%
for nFile = 1:numel(tifFileNameAll)    
    imgFile = fullfile(folder, tifFileNameAll{nFile});
    maskObjFile = fullfile(folder, maskFileNameAll{nFile});    
    removeBgUseMaskFun(imgFile, maskObjFile, pages, tifSuffix);
end


%%


function removeBgUseMaskFun(imgFile, maskObjFile, pages, tifSuffix, r, isFilterBrightOutlier, isOutlierLargeArea)
%preprocess.cleanBackground(): Remove the background and outlier object in the imgFile, the background noise in the region outside the interested object is
%also removed as those pixels are set to 0
%Input: 
% imgFile:   includes the full path and the file name a single/multiple frame img3D file, e.g. a psfFile or mfm captured single/multiple frame insuline/bacteria file
%       e.g.  imgFile =  fullfile('..', '..', '..',  'smallWorlds', '20170419_Insulin_PSFs', 'PSF_1_f204.tif');
% maskObjFile: single frame or multiple frame mask file, with positive value (1 for binary, 255 for uint8) for region of objects we care (such as psf or bacteria).  
%           It sets the binary mask maskObj3D.
%           If it is multiple frame tif file, we read it and maskObj3D = (maskObjFile.Read) > 0, the 3D binary mask with interested object (with background)
%           if it is 2D, We repeat this mask along z to generate maskObj3D
% EMGain:   
% pages:    opitional: we may only process the speficied pages in the imgFile, as the top and bottom pages may not be used anyway!
% r:        radius of the Gaussisan weighted mask, default is 80, pixel with distance r has 10% weight as pixel with distance 0
% isFilterBrightOutlier:   true /false (default) whether remove bright outlier outside the mask
%                       2017.12.05, set default to false, as it seems to be buggy for psf clean
% isOutlierLargeArea:  true/false(default), 
%                      flase for psf, true for insulin object image, as bright outlier outside interested psf are
%                      ioslated pixels (hot pixel etc), but outlier outside insulin maybe area of pixels (unwanted cell bleach)
% 
%Output: 
% write the following .tif files in the same folder as imgFile with following suffixes added to imgFile:
% '*_maskBrightOutlierOutsideObj3D.tif':  maskBrightOutlierOutsideObj3D write as uint8 tif (with pixel value either 0 or 255), 
%                       maskBrightOutlierOutsideObj3D is 3D mask of outliers pixels outside the object.
%                       Outlier pixels are pixels not only has background, but other strange signals, they are typically too bright
%                           (used to be binary tif, but now uint8 tif, because imageJ can't read binary multiple pages .tif files)
% '*_maskPureBgOutsideObj3D.tif':     maskPureBgOutsideObj3D write as uint8 tif
%                       maskPureBgOutsideObj3D is 3D mask of pure background pixels outside the object
% '*_objMaskPortion.tif':       copy of img3D, with all pixel inside the maskObj3D set to 0
%               note, the camera pixel offset (around 491.5) is NOT removed from pixel values when save _bg.tif
% '*_bg.tif':   bgFile, the uint16 tif that bg3D write to, pure background for all the pixels in the image
%               note, the camera offset (around 491.5) is NOT removed from pixel values when save _bg.tif
%                       (1) pixels inside the maskPureBgOutsideObj3D have values copied from img3D (pure exact background + noise) 
%                       (2) pixels outside the maskPureBgOutsideObj3D have values computed from nearby pixels inside the maskPureBgOutsideObj3D (estimated background, but not noise)
%                           Note that (2) includes both pixels inside object mask, and pixels outside object maxk, the latter
%                           are bright outlier pixels
% '*_objWithoutBgNonNeg.tif':  img3D - bg3DwithOutlier, with pixels outside maskObj3D set to 0 and negative pixel values set to 0
%                       2017.10.09 May do Hack for insuline, just use the median inside each mask, as the outside too much
% Usage:
%   preprocess.cleanBackground();
%
% NOTE:
%       We divide mask outside object into two parts: pixels with pure background, and pixels has bright outliers (plus
%       background), i.e.:
%            ~maskObj3D = maskPureBgOutsideObj3D | maskBrightOutlierOutsideObj3D
%
% History:
%   2017.10.04_remove bright outlier outside the mask
%
% Todo: pixel dependent auto-computed r, for a given s (the number of pixels need to be included to compute the background, such as a number from 100~1000)

%% process inputs
% set imgFile, imgFilePath, imgFileNameNoExt
if ~exist('imgFile', 'var') || isempty(imgFile)
    [imgFileName, imgFilePath] = uigetfile('*.tif',' select img file');
    [~, imgFileNameNoExt] = fileparts(imgFileName);
    imgFile = fullfile(imgFilePath, imgFileName);
else
    [imgFilePath, imgFileNameNoExt] = fileparts(imgFile);
end

% set maskObjFile
if ~exist('maskObjFile', 'var') || isempty(maskObjFile)
    maskObjFile = fullfile(imgFilePath, [imgFileNameNoExt, '_mask.bmp']); % default maskObjFile
    if ~exist(maskObjFile, 'file')
        [maskFileName, maskFilePath] = uigetfile(fullfile(imgFilePath, '*.bmp'),' select mask file for img');
        maskObjFile = fullfile(maskFilePath, maskFileName);                
    end
end

if ~exist('EMGain', 'var') || isempty(EMGain)
    EMGain = 1000;
end

% set pages
fileInfo = imfinfo(imgFile);   % return tiff structure, one element per page
if ~exist('pages', 'var') || isempty(pages)
    % Default, read all pages.
    pages = 1:numel(fileInfo);
else
    % User specified pages, verify if they are the subset of 1:numel(tiffFileInfo)
    assert(all(ismember(pages, 1:numel(fileInfo))), sprintf('User specified pages are not subset of the valid page from 1 to %d', numel(fileInfo)));
end



if ~exist('r', 'var') || isempty(r)
    r = 80; % for each pixel, only search window with (2r+1)*(2r+1) is enough, with much less computation v.s. search whole image of 1024*1024;
end


if ~exist('isFilterBrightOutlier', 'var') || isempty(isFilterBrightOutlier)
    isFilterBrightOutlier = false;
end

if ~exist('isOutlierLargeArea', 'var') || isempty(isOutlierLargeArea)
    isOutlierLargeArea = false;
end



%% Read img, which could be either 2D or 3D
PRECISION = 'single';
img3D = cast(tif.read(imgFile, pages), PRECISION);
[Ny, Nx, Nz] = size(img3D);

%% Read mask/ maskObj3D, 2017.12.04 updated to handle 3D mask file
switch lower(maskObjFile(end-2:end))
    case {'bmp', 'png'}
        mask = imread(maskObjFile); mask = mask>0; % read and set the binary 2D /3D mask
        %assert(ndims(mask)==2, sprintf('Mask is %dD. Currently only handle 2D mask!', ndims(mask)));
        assert( all(size(mask)==[Ny, Nx]), sprintf('Image and Mask has different heightxwidth: %dx%d vs %dx%d', Ny, Nx, size(mask,1), size(mask,2)));
        maskObj3D = repmat(mask, [1, 1, Nz]);
    case 'tif'
        % 3D mask
        maskObj3D = tif.read(maskObjFile, pages); maskObj3D = maskObj3D > 0;  % 
        assert(all(size(maskObj3D)==[Ny, Nx, Nz]), sprintf('Image and Mask has different size  %dx%dx%d vs %dx%dx%d', Ny, Nx, Nz, size(maskObj3D,1), size(maskObj3D,2),  size(maskObj3D,3)));
    otherwise
        error('unkown file format %s', lower(maskObjFile(end-2:end)));
    
end


%% Remove background/unwanted signal outside mask: set to 0 after removal
img3DZeroOutMask = img3D; img3DZeroOutMask(~maskObj3D) = 0;
tif.write(uint16(img3DZeroOutMask), fullfile(imgFilePath, [imgFileNameNoExt, '_objMaskPortion.tif']));


%% Remove background inside maskObj3D (object mask) with two steps
%% (I) Compute maskPureBgOutsideObj3D , the pure background outside object mask and corresponding (background image). Normally ~maskObj3D ( inverse of object mask) is the bg3D
% But in some cases, the ~maskObj3D has some outlier object, such as another cell at the boundary of tiles in 20170419_Insulin_PSFs Dataset
% maskPureBgOutsideObj3D = ~maskObj3D & ~maskBrightOutlierOutsideObj3D
maskPureBgOutsideObj3D = ~maskObj3D;
if isFilterBrightOutlier
    % TODO: 2018.05.21 and later, the input file is already in photon units, so need to scale below correct
    cam = CAMERA('EMGain', EMGain);
    for iter = 1:2
        % two-step refinement, as bgMedian may not be accurate at iter 1 (20170419_Insulin_PSFs Dataset only a few pixels difference, and is ok)
        bgMedian = zeros(1,Nz); %bgMedian = median(reshape(img3D(maskPureBgOutsideObj3D), [], Nz), 1) is wrong, can't handle when maskPureBgOutsideObj3D is different at different frame
        for n = 1:Nz
            curImg = img3D(:,:,n);
            bgMedian(n) = median(curImg(maskPureBgOutsideObj3D(:,:,n)));
        end
        bgStd = sqrt(cam.gaussianStdDU^2 + (bgMedian-cam.camOffSetConstant)*cam.gain); % total std of Gaussian + Poisson noise, about 205 for insulin, mostly from Poisson
        maskBrightOutlierOutsideObj3D = bsxfun(@ge, img3D, reshape(bgMedian+bgStd*2, 1, 1, Nz)); % 2*std covers 95% of Gaussian, what about Poisson?
        maskBrightOutlierOutsideObj3D(maskObj3D) = 0;
        maskPureBgOutsideObj3D = ~maskObj3D & ~maskBrightOutlierOutsideObj3D;
    end
    if isOutlierLargeArea
        se = strel('disk',10); %11x11
        for z = 1:Nz
            maskBrightOutlierOutsideObj3D(:,:,z) = bwareaopen(maskBrightOutlierOutsideObj3D(:,:,z), 5); % remove all connected components (objects) that have fewer than 5 pixels
            maskBrightOutlierOutsideObj3D(:,:,z) = imclose(maskBrightOutlierOutsideObj3D(:,:,z), se);
        end
    end    
else
    maskBrightOutlierOutsideObj3D = false(size(img3D));
end


maskPureBgOutsideObj3D = ~maskObj3D & ~maskBrightOutlierOutsideObj3D;

tif.write(uint8(uint8(maskBrightOutlierOutsideObj3D)*uint8(255)), fullfile(imgFilePath, [imgFileNameNoExt, '_maskBrightOutlierOutsideObj3D.tif'])); % write as uint8 rather than binary multiple page tif as imageJ fails to read the later
tif.write(uint8(uint8(maskPureBgOutsideObj3D)*uint8(255)), fullfile(imgFilePath, [imgFileNameNoExt, '_maskPureBgOutsideObj3D.tif']));


%% Increase the radius r if it is too small
for z = 1:Nz
    D = bwdist(maskPureBgOutsideObj3D(:,:,z)); % Distance transform: a number that is the distance between that pixel and the nearest nonzero pixel of BW.
    r = max(r, double(ceil(max(D(:))))+10); % or 100 is good for 20170419_Insulin_PSFs    
end
fprintf('kernel radius used: r = %d.\n', r);

%% (II) for each pixel p inside mask, set the background level the same as the average of 100 ~ r^2 closest background pixels outside the mask 
% This involves compute the bg3D, the background level 
% CPU Time: filter one z slice using radius r = 100 takes 25 seconds. r = 80 takes 15 seconds, 
%           filter all 200 slices use r = 80 takes 1500 secons = 25 minutes (by vectorize code to avoid the loop of 200 slices)
% changing from r = 80 to r = 100, intensity difference very little

% Generate Gaussian weighted mask inside a circle, first generate a squared mask, then set all weights outside the circle as 0
sigma = r/sqrt(2*log(1/0.1)); % sigma = 0.46*r = 37.2
h = fspecial('gaussian', 2*r+1, sigma); % (2r+1)*(2r+1) square, must be odd size
[~, yxOutCircle] = shape.getCoordsInCircle(r, [r+1, r+1]); % all interger coordinates OUTSIDE circle with radius r and centered at (r+1,r+1);
idxOutCircle = sub2ind(size(h), yxOutCircle(:,1), yxOutCircle(:,2));
h(idxOutCircle) = 0;
hMask = h>eps;


bgFile = fullfile(imgFilePath, [imgFileNameNoExt, '_bg.tif']);
if exist(bgFile, 'file')
    answer = questdlg('Background file already computed before, do you want to compute again (takes a while)?', 'Warning!', 'Yes', 'No', 'No');
end
if exist(bgFile, 'file') && (strcmp(answer, 'No') || isempty(answer))         
    % If background file exists, and user don't want to compute again, then just load it
    bg3D = cast(tif.read(bgFile), 'like', img3D);
else
    % If background file doesn't exist, or exist and user want to compute again, then compute the background for maskObj3D region
    bg3D = zeros(size(img3D), 'like', img3D); 
    bg3D(maskPureBgOutsideObj3D) = img3D(maskPureBgOutsideObj3D);
        
    fft2DSz = size(h) + [Ny, Nx] -1; 
    % pad the weight kernel, then circularshift the center from (r+1,r+1) to (1,1) when kernel are all zero with pixel (1,1) = 1; it is the delta function.    
    H = fftn(circshift(padarray(double(h), fft2DSz-size(h), 0, 'post'), [-r, -r]));
    HMask = fftn(circshift(padarray(double(hMask), fft2DSz-size(hMask), 0, 'post'), [-r, -r]));

    for z = 1:Nz
        fprintf('Processing frame %d out of %d frames.\n', z, Nz);
        maskBgFFT = fftn(double(maskPureBgOutsideObj3D(:,:,z)), fft2DSz);
        bgFFT = fftn(double(bg3D(:,:,z)), fft2DSz);        
        
        bgCounts = ifftn(maskBgFFT.*HMask); bgCounts = bgCounts(1:Ny, 1:Nx);  % same result but faster than  bgCounts = conv2(maskBg, hMask, 'same');
        MIN_NO_PIXEL = 100;
        minCounts = min(bgCounts(:)); % minCounts = min(bgCounts(maskObj3D(:,:,z))); % may change to only consider bgCounts inside the cared object mask
        %assert(nnz(maskBgW)>=MIN_NO_PIXEL, sprintf('The numbel of nearby pixels outside the mask %d, which is smaller than %d', nnz(maskBgW), MIN_NO_PIXEL));
        if minCounts < 1
            error('Zero number of nearby background pixels within radius %d, MUST increase the search radius.', r)
        elseif minCounts < MIN_NO_PIXEL
            warning('%d (<%d) Number of nearby background pixels within radius %d, suggest increase the search radius.', minCounts, MIN_NO_PIXEL, r);
        end
                
        bg2 = ifftn(bgFFT.*H);              bg2 = bg2(1:Ny, 1:Nx);  % same result but faster than bg2 = conv2(bg, h, 'same');
        bgWeights = ifftn(maskBgFFT.*H);    bgWeights = bgWeights(1:Ny, 1:Nx);  % same result but faster than bgWeights = conv2(maskBg, h, 'same');                                        
        % set the background level for both pixels inside the maskObj3D,  and also set a dummy background level for pixels
        % outside the maskObj3D, though only need assign inside the maskObj3D portion, it's easier to write code to assign whole image here                
        bg3D(:,:,z) = bg2 ./ bgWeights; 
    end
    
    %% Trash Old code without fft, at least 10 times slower for img3D of size 1024*1024*200, even with carefull code vectorization
    isOldCode = false;
    if isOldCode         
        idxAll = find(mask);
        [yAll, xAll] = ind2sub(size(mask), idxAll);
        bg3D0 = img3D; bg3D0(~maskPureBgOutsideObj3D) = 0;
        z = 1:Nz; % z= 1; %test with z = 1
        for n = 1:numel(yAll)
            % display the bar
            printIncrease = round(numel(yAll)/100);
            percentage = n/printIncrease;
            if percentage == round(percentage)
                fprintf('Finished %%%d.\n', percentage);
            end
            y = yAll(n);
            x = xAll(n);
            yIdx = max(y-r,1) : min(y+r, Ny);
            xIdx = max(x-r,1) : min(x+r, Nx);
            IW = img3D(yIdx, xIdx, z);
            maskBg3DW = maskPureBgOutsideObj3D(yIdx, xIdx, z);
            maskBg = maskPureBgOutsideObj3D(:,:,1);
            maskBgW = maskBg(yIdx, xIdx);
            MIN_NO_PIXEL = 100;
            %assert(nnz(maskBgW)>=MIN_NO_PIXEL, sprintf('The numbel of nearby pixels outside the mask %d, which is smaller than %d', nnz(maskBgW), MIN_NO_PIXEL));
            if nnz(maskBgW)<MIN_NO_PIXEL
                warning('The numbel of nearby pixels outside the mask %d, which is smaller than %d', nnz(maskBgW), MIN_NO_PIXEL);
            end
            hMasked = h(maskBgW);
            hMasked = hMasked/sum(hMasked(:));
            bg3D0(y,x,z) =  sum(bsxfun(@times, reshape(IW(maskBg3DW), [], numel(z)), hMasked), 1); % 1 * Nz vector
        end
    end    
end
bg3D(maskPureBgOutsideObj3D) = img3D(maskPureBgOutsideObj3D); % 
% bg3D(~maskPureBgOutsideObj3D & ~maskObj3D) = 0; % keep the region of ~maskPureBgOutsideObj3D & ~maskObj3D as background
tif.write(uint16(round(bg3D)), bgFile);


imgObjWithoutBg3D = zeros(size(img3D), 'like', img3D);
imgObjWithoutBg3D(maskObj3D) = img3D(maskObj3D) - bg3D(maskObj3D); 

%%
tif.write(imgObjWithoutBg3D, fullfile(imgFilePath,  [imgFileNameNoExt,  tifSuffix '.tif']));
%imgObjWithoutBgNonNeg3D = imgObjWithoutBg3D; imgObjWithoutBgNonNeg3D(imgObjWithoutBgNonNeg3D<0) = 0;
%tif.write(uint16(round(imgObjWithoutBgNonNeg3D)), fullfile(imgFilePath,  [imgFileNameNoExt, '_objWithoutBgNonNeg.tif']));


end
