%2018.05.20, remove constant bg for each tile, 
% assume tiles are aligned blocks, compute bg as the mean of pixels at the border of each tile.

%% 20180516 data

folder = '/sandbox/smallWorlds/20180516_Peri_PSF_MFM3x3_Confocal';
tifFileNameAll = {'PSF_f201_PhU_GsStd1.84_fs', ...                    
                    'peri_1_PhU_GsStd0.27_wSum_fs', ...
                    'floodedIris_MFM_f100_fs', ...
                    'floodedIris_MFM_f100_sum_fs'};
pages = [];
tilesNoGutter = load(fullfile(folder, 'MFM_maskTiles.mat')); tilesNoGutter = tilesNoGutter.tilesNoGutter;

tifSuffix = '_constBgEachTileRemoved';
w = 10; %compute 10 pixels along each tile border

%%
for nFile = 1:numel(tifFileNameAll)
    tifFileName = tifFileNameAll{nFile};
    removeConstantBgEachTileFun(folder, tifFileName, pages, tifSuffix, tilesNoGutter, w)
end


%%
function removeConstantBgEachTileFun(folder, tifFileName, pages, tifSuffix, tilesNoGutter, w)        
    %compute w pixels along each tile border
    if ~exist('w', 'var') || isempty(w)
        w = 10; % w = 300/30
    end
    tifFile = fullfile(folder, [tifFileName, '.tif']);
    imgAll = tif.read(tifFile, pages);    
    %%            
    fprintf('Remove a constant tile/frame dependent background for each tile and each frame tifFile %s (%d pages).\n', ...
        tifFileName, size(imgAll,3));
    [Ny, Nx, Nz] = size(imgAll);
    winIdx = [-tilesNoGutter.winHf:tilesNoGutter.winHf];
    winIdxNoBorder = [(-tilesNoGutter.winHf+w): (tilesNoGutter.winHf-w)];
    for nTile = 1:numel(tilesNoGutter.ycxcList)
        yc = tilesNoGutter.ycxcList{nTile}(1);
        xc = tilesNoGutter.ycxcList{nTile}(2);
        tileMask = false(Ny, Nx);
        tileMask(yc+winIdx, xc+winIdx) = true;
        tileMaskNoBorder = false(Ny, Nx);
        tileMaskNoBorder(yc+winIdxNoBorder, xc+winIdxNoBorder) = true;        
        tileBorder = tileMask & ~tileMaskNoBorder;            
        for nz = 1:Nz
            img = imgAll(:,:,nz);
            img(tileMask) = img(tileMask) - mean(img(tileBorder));
            imgAll(:,:,nz) = img;
        end
    end
        
    %%                    
    tifBgRemoved = fullfile(folder, [tifFileName, tifSuffix '.tif']);
    tif.write(imgAll, tifBgRemoved);
    
        
end