function [yx, yxOut] = getCoordsInCircle(r, yx0)
%GETCOORDSINCIRCLE get the interger/pixel coordintates inside (include boundary) a circle with radius r and center yx0
% Inputs:
%   r:    radius of circle, need to be >0
%   yx0:  the center of the radius is (y0, x0) = (yx0(1), yx0(2))
% Outputs:
%   yx:   n*2 matrix, where (y,x) = (yx(:,1), yx(:,2)), such that
%           (y-y0).^2 + (x-x0).^2 <= r^2
%   yxOut: pixels outside the mask
if nargin == 0
    testMe();
    return
end

if ~exist('yx0', 'var') || isempty(yx0)
    yx0 = [0, 0];
end

y0 = yx0(1); 
x0 = yx0(2);

[Y, X] = ndgrid(ceil(y0-r):floor(y0+r),  ceil(x0-r):floor(x0+r));

mask = (Y-y0).^2 + (X-x0).^2 <= r^2;

yx = [Y(mask), X(mask)];
if nargout == 2
    yxOut = [Y(~mask), X(~mask)];
end
end


function testMe()
N= 4;
rAll = [5; rand(N-1,1)*10] ;
yx0All = [0, 0; rand(N-1,2)*5];
figure; clf; 
for n = 1:N
    r = rAll(n);
    yx0 = yx0All(n,:);
    yx = shape.getCoordsInCircle(r, yx0);
    subplot(ceil(sqrt(N)), ceil(sqrt(N)), n); hold on;
    viscircles(reverse(yx0), r, 'color', 'b');  %xy0 = reverse(yx0);
    plot(yx(:,2), yx(:,1), 'xr'); 
    xlabel('x'); ylabel('y'); axis equal; grid on; grid minor; title(sprintf('radius: %.2e, center: (%.2e, %.2e)', r, yx0(2), yx0(1)));
end
end

% Different algorithm: loop  is to loop, which is fater in c/python but slower in matlab 
%{
y0 = yx0(1);
x0 = yx0(2);
yx = [];
for x = ceil(x0-r) : floor(x0+r)
    d = sqrt(r^2 - (x-x0).^2);
    y = (ceil(y0-d) : floor(y0+d))';
    yx = [yx; y x*ones(size(y))];
end
%}
