function S2 = copy(S, S2, excludedFields, verbose)
%struct.copy(S, S2, verbose); overloaded function for struct: copy all the filed in S to field in S2.
% if S is a structure array, S2 will be a structure array
% TODO: if S is a structure array, currently assume S2 is not provided, in future if S2 is provided, check it
% (1) If S2 has extra fields, then those fileds are kept
% (2) If a field was originally in S but not S2, then this field will be appear in S2, with value copied from S. 
%  verbose: opitional, default is false. If it is true, we display a waring in case (2)

%  excludedFields: we don't want to copy any of S's fields that within excludedFileds
% For example, we may want to exclude a field f it is too large and not good to to on disk
% It could happen if the field is a large array, or a function handle that implicitly requires a large array as function paramaters
% TODO, convert function handle fields to string even if it is in the exludedFiedlds

% 2017.06.14: Xiang Huang
if nargin == 0
    testMe();
    return
end

if ~exist('S2', 'var') || isempty(S2) 
    S2 = repmat(struct(), size(S)); % handle both cases, S is a single struture element or a structure array 
    verbose = false;
end

if ~exist('excludedFields', 'var') || isempty(excludedFields)
    excludedFields = {};
end


if ~exist('verbose', 'var') || isempty(verbose)
    verbose = true;
end

%fieldname
excludedStr = 'Not saved: too large array or function handle involes other classes, arraysthat is too large';
fnAll = fieldnames(S(1));
for n = 1:numel(fnAll)
    fn = fnAll{n};
    if verbose && ~isfield(S2, fn)
        warning('Creating a non-exist field %s in destination struct', fn);
    end
    if ~any(strcmp(fn, excludedFields))
        [S2.(fn)] = S.(fn); % handle both cases, S is a single struture element or a structure array
    else
        tmp = repmat({excludedStr}, size(S)); [S2.(fn)] = tmp{:}; % use cell array
        %[S2.(fn)] = deal(excludedStr); % use deal(excludedStr) 
    end
    
end

end


function testMe()
%% 1. Copy s to a empty structure
fprintf('----------S-----------\n');
S = struct('f1', 4, 'f2', 5);
display(S);

fprintf('----------S2 (empty or not exist before copy) copied from S-----------\n');
S2 = struct.copy(S);
display(S2);


%% 1.2 Copy s to a empty structure with some fields excluded
fprintf('----------S-----------\n');
S = struct('f1', 4, 'f2', 5);
display(S);

fprintf('----------S2 (empty or not exist before copy) copied from S, with field f1 excluded-----------\n');
S2 = struct.copy(S, [], {'f1'});
display(S2)

%% 2.Copy s to S2, a structure has more fields
S = struct('f1', 4, 'f2', 5);
S2 = struct('f1', 1, 'f2', 2, 'f3', 3);
fprintf('----------S2 Before Copy ------------\n');
display(S2);
S2 = struct.copy(S, S2);
fprintf('----------S2 After Copy ------------\n');
display(S2);

%% 3. Copy a structure array to a empty structure
S = struct('f1', {1,11}, 'f2', {2, 22});
fprintf('----------S: a structure array-----------\n');
display(S);
display('S(1):'),
display(S(1)),
display('S(2):'),
display(S(2))

fprintf('----------S2 (empty or not exist before copy) copied from S-----------\n');
S2 = struct.copy(S);
display(S2);
display('S2(1):'),
display(S2(1)),
display('S(2):'),
display(S2(2))

%% 3.2 Copy a structure array to a empty structure
S = struct('f1', {1,11}, 'f2', {2, 22});
fprintf('----------S: a structure array-----------\n');
display(S);
display('S(1):'),
display(S(1)),
display('S(2):'),
display(S(2))

fprintf('----------S2 (empty or not exist before copy) copied from S, with field f1 excluded-----------\n');
S2 = struct.copy(S, [], 'f1');
display(S2);
display('S2(1):'),
display(S2(1)),
display('S2(2):'),
display(S2(2))



end

%{
fprintf('----------S-----------\n');
S = struct('f1', 4, 'f2', 5);
struct.display(S);

fprintf('----------S2 start from S,  copied from S2-----------\n');
S2 = struct.copy(S);
struct.display(S2);


S2 = struct('f1', 1, 'f2', 2, 'f3', 3);
fprintf('----------S2 Before Copy ------------\n');
struct.display(S2);
S2 = struct.copy(S, S2);
fprintf('----------S2 After Copy ------------\n');
struct.display(S2);
%}
