function bitCut(fileLoc)
%tif.bitCut(fileLoc) Convert grayscale tiff/fits files to lower bit tiff file without lossing information
%   e.g. if imgFile/fits is 32bit and all pixels < 2^16, we could convert it to 16bit tiff without lossing any information
% Inputs:
%   fileLoc: either a folder or a file with complete path
%   Xiang Huang: xianghuang@gmail.com
%   2017.03.08 Created to handle Xiaolei's random bitdepth files (some with 16 bit, some with 32bit but value < 2^16)

%% Parse input to find the pathStr and (optional) imgFile;
if ~exist('fileLoc', 'var') || isempty(fileLoc)
    % user interactively specify path
    pathStr = uigetdir();
elseif exist(fileLoc, 'dir')  % need to check dir before check file, and exist(tiffFile, 'file') will check for both file and dir
    pathStr = fileLoc;
elseif exist(fileLoc, 'file')
    pathStr = [];
    imgFile = fileLoc;    
else
    error('Invalid tiff location specified by: %s', fileLoc);
end

%% If pathStr not specified, use the specified imgFile, otherwise use all '*.tif' or '*.tiff" files in that path
if isempty(pathStr) % imgFile must exist
    imgFileAll = {imgFile};
else
    imgFileAllStruct = [dir(fullfile(pathStr, '*.tif')); dir(fullfile(pathStr, '*.tiff')); ir(fullfile(pathStr, '*.fits'))];
    imgFileAll = cellfun(@(x) fullfile(pathStr, x), {imgFileAllStruct.name}, 'UniformOutput', false);
end

%%
for n = 1:numel(imgFileAll)
    bitCutFile(imgFileAll{n});    
end
%%    

end

%%
function isBitCut = bitCutFile(imgFile)
%% cut any uncessary depth of tiff file and save it as orgi rename it to [imgFile '_name'] and save 
bitPossible = [1, 8, 16, 32, 64];
bits2Class = containers.Map(bitPossible, {'logical', 'uint8', 'uint16', 'uint32', 'uint64'});

[~, ~, ~] = fileparts(imgFile);
if strcmpi(ext, '.fits')
    imgAllPages = fitsread(imgFile); % fitsread always read to double floating point though all the values could be interger
else
    imgAllPages = tif.readShowWrite2JPGs(imgFile);
end

IMax = max(imgAllPages(:));
bit = bitPossible(find(log2(double(IMax)+1) <= bitPossible, 1));  % find the minimum candidate bit such that 2^bitDepth-1 >= IMax
newClass = bits2Class(bit);
originalClass = class(imgAllPages);

isBitCut = ~isa(imgAllPages, newClass);
if isBitCut
    [pathStr, fName, fExt] = fileparts(imgFile);
    subfolder =  sprintf('original_%s', originalClass); 
    subfolderFullPath = [pathStr filesep subfolder filesep];
    fprintf('Convert from %s to %s for ''%s''. Keep the original in subfolder ''%s.''\n',  originalClass, newClass, imgFile, subfolder);
    imgAllPages2 = cast(imgAllPages, newClass);    
    if ~exist(subfolderFullPath, 'dir')
        mkdir(pathStr, subfolder);  %mkdir will ignore if subfolder exists
    end
    movefile(imgFile, subfolderFullPath); % movefile will create a folder if not exist
    tif.write(imgAllPages2, imgFile);
else
    fprintf('No change to %s file: ''%s''.\n', originalClass, imgFile);
end



end

