function [imgAll_zxy, imgAll_zyx] = conver2yxSlices(tiffFile)
%tif.conver2yxSlices(tiffFile)
% read a tiffFile, which is 3D volume with z slicing (pages along z), permuate the 3D volume and write to .tif file with pages along x, and y
%
%
% Input:
%   tiffFile or imgAllPages:  
%       tiffFile: location of multipage tiff image file, including the path. e.g. 'C:\Dropbox\smallworlds\data\MFM_PSF.tif'
%           future handle 4D for 3D/4D matrix. Gray scale image with one (2D) or multiple pages (3D), or rgb image with multiple pages   (4D)
%                 binary/uint8/uint16/single-float/double-float format
% Output:
%       None
% 
% 2018.05.21 added


if ~exist('tiffFile', 'var') || isempty(tiffFile)
    [filename, pathStr] = uigetfile('*.tif',' select tiff image stack file');
    tiffFile = fullfile(pathStr, filename);    
end

[pathStr, fileName, fileExt] = fileparts(tiffFile);

imgAll = tif.read(tiffFile);

pageDim = ndims(imgAll);
assert(pageDim >= 3, 'must be multiple page fil');

imgAll_zxy = permute(imgAll, [3, 2, 1]);
imgAll_zyx = permute(imgAll, [3, 1, 2]);
tif.write(imgAll_zxy, fullfile(pathStr, [fileName, '_zxy', fileExt]));
tif.write(imgAll_zyx, fullfile(pathStr, [fileName, '_zyx', fileExt]));

end
