function fits2tiff(fitsLoc)
%tif.fits2tiff Convert a fits file to a tiff file
%   Detailed explanation goes here
%bitCut Convert grayscale tiff files to lower bit without lossing information
%   e.g. if fitsFile is 32bit and all pixels < 2^16, we could convert it to 16bit tiff without lossing any information
% Inputs:
%   fitsLoc: either a folder or a file with complete path
%   Xiang Huang: xianghuang@gmail.com
%   2017.03.16 Created to handle Xiaolei's new format .fits (some with 16 bit, some with 32bit but value < 2^16)


%% Parse input to find the pathStr and (optional) fitsFile;
if ~exist('fitsLoc', 'var') || isempty(fitsLoc)
    % user interactively specify path
    pathStr = uigetdir();
elseif exist(fitsLoc, 'dir') % need to check dir before check file, and exist(fitsFile, 'file') will check for both file and dir
    pathStr = fitsLoc;
elseif exist(fitsLoc, 'file')
    pathStr = [];
    fitsFile = fitsLoc;
else
    error('Invalid tiff location specified by: %s', fitsLoc);
end

%% If pathStr not specified, use the specified fitsFile, otherwise use all '*.tif' or '*.tiff" files in that path
if isempty(pathStr) % fitsFile must exist
    fitsFileAll = {fitsFile};
else
    fitsFileAllStruct = dir(fullfile(pathStr, '*.fits'));
    fitsFileAll = cellfun(@(x) fullfile(pathStr, x), {fitsFileAllStruct.name}, 'UniformOutput', false);
end

%%
for n = 1:numel(fitsFileAll)
    fits2tiffFile(fitsFileAll{n});
end
%%

end

%%
function fits2tiffFile(fitsFile)
%% cut any uncessary depth of tiff file and save it as orgi rename it to [fitsFile '_name'] and save
bitPossible = [1, 8, 16, 32, 64];
bits2Class = containers.Map(bitPossible, {'logical', 'uint8', 'uint16', 'uint32', 'uint64'});

imgAllPages = fitsread(fitsFile);

IMax = max(imgAllPages(:));
bit = bitPossible(find(log2(double(IMax)+1) <= bitPossible, 1));  % find the minimum candidate bit such that 2^bitDepth-1 >= IMax
newClass = bits2Class(bit);

[pathStr, fName, fExt] = fileparts(fitsFile);
tiffFile = fullfile(pathStr, [fName, '.tif']);
if ~exist(tiffFile, 'file')    
    subfolder =  sprintf('original_fits');
    subfolderFullPath = [pathStr filesep subfolder filesep];
    if ~exist(subfolderFullPath, 'dir')
        mkdir(pathStr, subfolder);  %mkdir will ignore if subfolder exists
    end    
    fprintf('Convert from fits to tif for ''%s''. Keep the original in subfolder ''%s.''\n',  fitsFile, subfolder);
    imgAllPages2 = cast(imgAllPages, newClass);    
    movefile(fitsFile, subfolderFullPath); % movefile will create a folder if not exist
    tif.write(imgAllPages2, tiffFile);
else
    fprintf('%s already exist, ignore.\n', tiffFile);
end



end
