function fileInfo = parseName(fileName)
%tif.parseName Find the EMGain, exposure, illumination, frameNo settings from the fileName
%   Usage:
%   fileInfo = tif.parseName('bg_I10ex400em1000_shutterOn.tif')
% results:
% fileInfo = 
%   struct with fields:
% 
%           EMGain: 1000
%         exposure: 400
%     illumination: 10
%          frameNo: []
%   fileInfo = tif.parseName('20170314_camOffset_ex400em700f100.tif'), fileInfo = parse('20170314_camOffset_ex400em700f100.fits')
% both results:
% fileInfo = 
%   struct with fields:
% 
%           EMGain: 700
%         exposure: 400
%     illumination: []
%          frameNo: 100
% NOTE: currently only handle integer numbers

%%  fileInfo is a structure contains: exposure, EMGain, illumination, frameNo
[pathstr, fName, fExt] = fileparts(fileName);
fName = lower(fName);
%%
fileInfo = struct('EMGain', [], 'exposure', [], 'illumination', [], 'frameNo', []);
fn = fieldnames(fileInfo);
field2Tag = containers.Map(fn, {'em', 'ex', 'i', 'f'});
%%
for n = 1:numel(fn)
    field = fn{n};
    tag = field2Tag(field);
    retStrCell = regexp(fName, [tag '[0-9]+'], 'match'); % 'em[0-9]+' etc, currently only handle integer numbers
    if numel(retStrCell) >= 1
        fileInfo.(field) = str2double(retStrCell{1}(numel(tag)+1:end));
        if numel(retStrCell) > 1
            warning('find multiple matches: %s. Use the first one %s\n.', sprintf('%s, ', retStrCell{:}), retStrCell{1})
        end
    end
end
end


