
%% FFT speed  
% 101 computation of fft2 of size 505^2 vs 512^2 vs 1024^2:  1.0 vs 2.0 vs 3.5 seconds
%% Prepare data
% 1. random psf
% rng(0, 'twister'); fSz = round(rand(1,3)*20)*2 + 4; %fSz = [4, 4, 3];
% rng(0, 'twister'); gSz = fSz(1:2) + round(rand(1,2)*20)*2;%gSz = [6, 6];
% psfSz = [gSz(1), gSz(2), fz]; %[1024, 1024, 101]; %[3, 4, 5]; [13, 13, 5]; [33, 33, 7];
% rng(0, 'twister'); psf = randn(psfSz); tSz = 1;
% 2. Gaussian tiled psf
tSz = [3, 3]; 
psfSz = [11*tSz, 9];
[psf, tSz]  = simulatePSF(psfSz, tSz, 1);
psfSz = size(psf);
gSz = psfSz(1:2);
fSz = [101, 101, psfSz(3)];
fc = ceil((fSz+1)/2);
% circular convolution: 
OTF = zeros(psfSz);
for z = 1:size(psf,3)    
    OTF(:,:,z) = fft2(psf(:,:,z)); %-median(Rz(:))); % move the origin to the top-left corner
end
% linear convolution
oSz = [psfSz(1:2)+fSz(1:2)-1, fSz(3)]; %oSz = [1024, 1024, fSz(3)];
OTFPadded = zeros(oSz);
for z = 1:fSz(3)  
    OTFPadded(:,:,z) = fftn(psf(:,:,z), oSz(1:2)); 
end

%% Compare new model with old model forward/backward/circular/linear, 2017.04.09_verified same
f = rand(fSz); 
psnr_=difference(MFMForward(f, OTF, gSz), MFMForwardOld(f, OTF, 'circular')); fprintf('psnr between old method and new method: %.3fdB.\n', psnr_);
psnr_=difference(MFMForward(f, OTFPadded), MFMForwardOld(f, OTFPadded, 'linear')); fprintf('psnr between old method and new method: %.3fdB.\n', psnr_);
g = rand(gSz);
psnr_=difference(MFMBackward(g, OTF, fSz), MFMBackwardOld(g, OTF, fSz, 'circular')); fprintf('psnr between old method and new method: %.3fdB.\n', psnr_);
psnr_=difference(MFMBackward(g, OTFPadded), MFMBackwardOld(g, OTFPadded, fSz, 'linear')); fprintf('psnr between old method and new method: %.3fdB.\n', psnr_);

%% Test circular convolution: Forward
f = zeros(fSz); f(fc(1), fc(2), fc(3)) = 1; g = MFMForward(f, OTF, gSz);
z=fc(3); figure(11),subplot(121); imagesc(psf(:,:,z));  axis image; subplot(122); imagesc(g);  axis image; difference(psf(:,:,z),g)
yxShift = floor(psfSz(1:2)./tSz/2)-1; f = zeros(fSz); f(fc(1)+yxShift(1), fc(2)+yxShift(2), fc(3)) = 1; g = MFMForward(f, OTF, gSz);
z=fc(3); figure(12),subplot(121); imagesc(psf(:,:,z));  axis image; subplot(122); imagesc(g);  axis image;  difference(circshift(psf(:,:,z), yxShift),g)


%% Test linear convolution Forward
f = zeros(fSz); f(fc(1), fc(2), fc(3)) = 1; g = MFMForward(f, OTFPadded, gSz); % g = MFMForward(f, OTFPadded); same
z=fc(3); figure(11),subplot(121); imagesc(psf(:,:,z));  axis image; subplot(122); imagesc(g);  axis image; difference(psf(:,:,z),g)
yxShift = floor(psfSz(1:2)./tSz/2)-1; f = zeros(fSz); f(fc(1)+yxShift(1), fc(2)+yxShift(2), fc(3)) = 1; g = MFMForward(f, OTFPadded, gSz); % g = MFMForward(f, OTFPadded) same
z=fc(3); figure(12),subplot(121); imagesc(circshift(psf(:,:,z), yxShift));  axis image; subplot(122); imagesc(g);  axis image;  difference(circshift(psf(:,:,z), yxShift),g)


%% Test circular convolution: Backward
%{
f = zeros(fSz); f(fc(1), fc(2), fc(3)) = 1; g = MFMForward(f, OTF, gSz);
fback1 = MFMBackward_CircConv(g, OTF, size(f));
%%
fback2 = MFMBackward(g, OTF, size(f), 'circular');
%%
fback3 = MFMBackward(g, OTFPadded);
%
%}

%% Verify function handles, make sure it generate same result as direct matrix multiplication
% 2017.04.09_verified for MFMForward/MFMBackward when fSz = psfSz;
gSz = [7, 7];
psfSz = [gSz, 5];
%fSz = [3, 4, psfSz(3)];
fSz = [3, 3, psfSz(3)];
rng(0, 'twister');  psf = rand(psfSz);
rng(0, 'twister');  f = randn(fSz); fPadded = padarray(f, (psfSz-fSz)/2, 0, 'both');
rng(0, 'twister');  g = randn(gSz);
OTF = zeros(psfSz);
for z = 1:size(psf,3)    
    OTF(:,:,z) = fft2(psf(:,:,z)); %-median(Rz(:))); % move the origin to the top-left corner
end
if prod(gSz)*prod(fSz)<1e8
    AMatrix = calcSensingMatrixViaPSF_CircConv(psf); % gSz(1)*gSz(2) by fy*fx*fz full matrix
    hA0 = @(fPadded) reshape(AMatrix*fPadded(:), gSz);
    AtFun0 = @(g) reshape(AMatrix'*g(:), size(fPadded));        
    psnr_ = difference(MFMForward(f, OTF, gSz), hA0(fPadded));
    fprintf('Verify forward AMatrix*f: function handle and matrix multiplication has difference in PNSR %.2fdB.\n ', psnr_);    
    rowIdx = floor((psfSz(1)-fSz(1))/2) + (1:fSz(1)); % only works when ySz-fSz(1:2)
    colIdx = floor((psfSz(2)-fSz(2))/2) + (1:fSz(2));    
    AtFun0y = AtFun0(g);
    psnr_ = difference(MFMBackward(g, OTF, fSz), AtFun0y(rowIdx, colIdx, :));
    fprintf('Verify conjugate transpose AMatrix''*g: functional handle and matrix multiplication has difference in PNSR %.2fdB.\n ', psnr_);       
end



