function [alpha, x2, obj_x2] = backTrackLineSearch(x, dx, gradxdx, objFun_xs, obj_x, theta, zeta, verbose)
% backtracking line search. Convex Optimization, byod, Page 464, but renamed alpha-->theta, beta-->zeta, s-->alpha
% alpha: step length
% zeta:  the backtracking shrinkage size of each step size.
% theta: percentage of reducement
% with additional domain requirement that  x >0  
% 2017.07.20: separate it as a function
% 2017.07.30: change from objFun(x2) to objFun(x2,alpha) for x2 = x+alpha*dx, to precompute (such as Adx for ||A(x+alpha*dx)-b||_2^2) to save computation time
% 2017.08.11: zeta 0.5--> 0.9 since we are running objFun_xs(x2, alpha) which is very fast as a function of alpha, we could afford more accurate zeta
if ~exist('theta', 'var') || isempty(theta)
    theta = 0.1;
end
if ~exist('zeta', 'var') || isempty(zeta)    
    zeta = 0.9; % 0.5 --> 0.9
end
if ~exist('verbose', 'var') || isempty(verbose)
    verbose = false;
end


%% Mainblock: 2017.09.08 change to two steps (strange, sometimes taking longest step alpha = 0.95 where x + alpha*dx>0, is not as good as old code of taking 0.9)
% I: find the largest alpha, such that alpha <=1 and x2 = x + alpha*dx > 0 for the given x >0
% 2017.09.22_modified need test
alpha = 0.999;
iNeg = dx < 0;
if nnz(iNeg) > 0
    alpha = min(alpha, 0.999*min(x(iNeg)./ (-dx(iNeg)))); % alpha = min(1, min(x(iNeg)./ (-dx(iNeg)))); % add 0.999 to make non negtaive
end
fprintf('--From x positive constraint: alpha=%.4e.\n', alpha);
% II: find the largest alpha = s0*zeta^k such that objFun_xs(x2, alpha) <= theta*alpha*gradxdx
threshold = obj_x + theta*alpha*gradxdx;
while objFun_xs(x + alpha*dx, alpha) > threshold
    alpha = alpha*zeta;
end
x2 = x + alpha*dx;


%% Output
if verbose && alpha < 1-eps
    fprintf('--From x positive and objective function value decrease certain amount: non-full stop (alpha<1) stepsize: alpha=%.4e.\n', alpha);
end
end


function trashbox()
%% Mainblock: Before the modification on 2017.09.08, the code below works
alpha = 1;
while true
    x2 = x + alpha*dx;
    if all(x2(:)>0)
        % make sure x is still in positive domain
        obj_x2 = objFun_xs(x2, alpha);
        if obj_x2 <=  obj_x + theta*alpha*gradxdx
            break
        end
%     else
%         if verbose
%             warning('negative: %d!\n', nnz(x2(:)<=0));
%         end
    end
    % either any of x2 element <= 0, or backtrack line search no good
    alpha = alpha*zeta; 
end
end