function [imgProj3Planes, maxProj] = calProjectedImages(f, voxSz)
% Find the maximum intensity when integrate a volumetric image f along three axis y/x/z, i.e., projection to x-z/y-z/y-x plane
% Input: 
%       f: 3D volumetric image
%          or a 1D/2D cell array of 3D volumetric images, 
%       voxSz:  the actual voxSz, such as [98, 98, 50]nm
%          or a 1D/2D cell array of multiple voxel sizes
% Output:
%       imgProj3Planes: cell array of three images, i.e., f projected to y-x, y-z, and x-z planes, i.e., intergrate along y/x/z axis
%       maxProj: the maximum intensity among three projected images
% Reference: 
%       auxiliary function for show3dPointsWithProjections()
% 2017.01.30: added handle multiple volumes 
%

% voxSz is always required, because the voxSz may never by cubic
if ~exist('voxSz', 'var') || isempty(voxSz)
    warning('Using Perfect Cubic voxSZ, are you sure?');
    voxSz = [1, 1, 1];
end

imgProj3Planes = cell(3,1); %y-x, y-z, and x-z plane; 
for k = 1:3
    imgProj3Planes{k} = squeeze(sum(f,k)*voxSz(k)); % be careful about integration with non-cuibic voxel-Size;  %imgProj3Planes{k} = squeeze(sum(f,k));
end

% maximum intentisy of three projections
maxProj = max(cellfun(@(x) max(x(:)), imgProj3Planes));
end

