function [bg, noiseStd] = calcBackgroundFrom4Corners(I, varargin)
%function [bg, noiseStd] = calcBackground(I, 'w', 11/101, 'verbose', true/false, 'corners', 4/36)
% 
% verify that we have xyz-spatial invariant background, return the background level and noise level
% Input:
%   I:  grayscale 2D or 3D image of size Isz
%   'w': window size for corners,default 101 (good for psf, but chose 11 for g)
%   'NCorner':  specify number of corners used for interpolation, such as 4 or 36 for 2x2 or 6x6 grids (in future specify the
%   corner coordinates instead);
%   'verbose': whether show intemediate figures
% Output:
%   bg: the intensity of constant background/ambient illumination for each frame
%   noiseStd:  the standard deviation of the noise
%
% 2017.04.24    Xiang Huang    new methods to estimate the background from single frame
% The psf background is different for each frame (z slice), compute the background for each frame
% For each frame: check 100*100 pixels in each of four corners, find the one with the minimum std (as psf is typically blurred more on two of the corners, avoid those corner), 
% that's the noiseStd, and the corresponding mean is the bg
% but nonono, we found one of the corner always has the 
% 2017.06.04:  this is not good enough!
% 2018.01.22:  this function is only kept as backup, as it is not good enough to remove background, use other methods!

warning('Using a DEPRECIATED INACCURATE method to cacluate the background from four 4 corners!');

%% Parse inputs
p = inputParser;  % CaseSensitive=0, PartialMatching=1
p.addParameter('w',         101,    @(x) isempty(x) || isscalar(x) && x>0 && x==round(x)); % positive integer
p.addParameter('NCorner',   4,      @(x) isempty(x) || isvector(x) && all(x>0) && all(x==round(x)) && ismember(x, [4, 36])); % positive interger: 2017.05.08_hack of either 4 or 36
p.addParameter('verbose',   false,  @(x) isempty(x) || isscalar(x) && ismember(x, [true, false, 1, 0])); % scalar true/false
p.parse(varargin{:});
cellfun(@(x) evalin('caller', sprintf('%s=p.Results.%s;', x, x)), fields(p.Results)); % e.g. eval('fSz = p.Results.fSz'); Or use in.fSz after set in =p.Results;

%%
[Ny, Nx, Nz] = size(I); % If I is 2D, we get Nz = 1

% check and found the four-corners of the I has almost the same distribution, with 5% maximum difference in mean
if NCorner == 4
    yx4Corners = cat(3, [1:w; 1:w], [1:w; Nx-w+1:Nx], [Ny-w+1:Ny; 1:w], [Ny-w+1:Ny; Nx-w+1:Nx]); %Left-Top, Right-Top, Left-Bottom, Right-Bottom
    I4Corners = zeros([4, w*w, Nz]); %[4, w, w, Nz]
elseif NCorner == 36
    fprintf('not ready!\n');
else
    error('unknown number of corners');
end
for i = 1:4
    I4Corners(i, :, :) = reshape(I(yx4Corners(1, :, i), yx4Corners(2, :, i), :), [w*w, Nz]);
end
%%
I4CornersMean = squeeze(mean(I4Corners, 2)); % 2 is the 2nd dimension of [4, w*w, Nz]
I4CornersStd = squeeze(std(I4Corners, 0, 2)); % 0 means normalize by w*w-1 instead of w*w, 2 is the 2nd dimension
I4CornersMedian = squeeze(median(I4Corners, 2)); % 2 is the 2nd dimension of [4, w*w, Nz]
[I4CornersStdMin, idx] = min(I4CornersStd, [], 1);
I4CornersMeanBest = I4CornersMean(idx+4*(0:Nz-1));
%%

% What is four 4-coners are not purely background + noise? but signal + background + noise?
% For I, we will see the ring effect, peak and valley at different depth, may shows in image as 'periodic' spikes
% For I, the intensity has almost the same mean along z-axis, which is a indicate that it has no signal, just background + hoise
legendAll = {'corner1:Left-Top', 'corner2:Right-Top', 'corner3:Left-Bottom', 'corner4:Right-Bottom'};
if verbose
    hf = figure('name', 'Mean & Std & Median vs Frame at 4 corners');
    subplot(221); plot(I4CornersMean'); title('Mean of 4 corners'); grid on; if(Nz>1); legend(legendAll); end
    subplot(222); plot(I4CornersStd');  title('Std of 4 corners'); grid on;  if(Nz>1); legend(legendAll); end
    subplot(223); plot(I4CornersMedian'); title('Median of 4 corners'); grid on;  if(Nz>1); legend(legendAll); end
   % subplot(223); plot(I4CornersMeanBest'); title('Mean of the corner with minimum Std'); 
   % subplot(224); plot(I4CornersStdMin'); title('Minimum Std of 4 corners');
    if ~verbose;  set(hf, 'Visible','off');  end
    if verbose;      printfig(['w=' int2str(w) '__' datestr(now, 'yyyymmdd_hhMM') '.png'], [3600, 2400]); end    
end

%% Bilinear Interpolation based on xy coordinates the median values: 
% wikipedia bilinear interpolations: 
%TODO: verify with other border values
yx4CornersCenter = squeeze(mean(yx4Corners, 2));  % 4 corner centers are [Q11, Q21, Q12, Q22] = [a b c d]
y1 = yx4CornersCenter(1, 1);
x1 = yx4CornersCenter(2, 1);
y2 = yx4CornersCenter(1, 4);
x2 = yx4CornersCenter(2, 4);

[y, x] = ndgrid(1:Ny, 1:Nx); % [Y, X] 2D grid
y = y(:); x = x(:);
scaler = 1/((x2-x1)*(y2-y1));

bg = (x2-x) .* (y2-y) * (scaler*I4CornersMedian(1,:)) + ...  % (Ny*Nx column vector)  *  (Nz row vector )
     (x-x1) .* (y2-y) * (scaler*I4CornersMedian(2,:)) + ...
     (x2-x) .* (y-y1) * (scaler*I4CornersMedian(3,:)) + ...     
     (x-x1) .* (y-y1) * (scaler*I4CornersMedian(4,:));
bg = reshape(bg, [Ny, Nx, Nz]);
noiseStd  = I4CornersStd;

%% Remove constant background along z
% psfMedianOverXY = zeros(pz, 1);
% for z = 1:pz
%     psfMedianOverXY(z) = median(reshape(I(:,:,z), [], 1));
% end
% figure, plot(psfMedianOverXY);