function [img, tilePhoton, imgPlusBg] = cleanBackgroundUseTiles(imgPlusOffsetBgUint16, tiles, cam, pages, retFolder, tag, Nbg)
% Inputs:
%       IUnit16 contains signal and background in raw uint16 format
%   Nbg: default 1000, the number of pixels in each tile that can be used to compute the background.
%
% Outputs: 
%       I: Masked by tile, Removed Non Uniform Cam Offset, normalized by gain, then each tile subtract constant background from mean of furthest 1000 pixels to center';

if ~exist('pages', 'var') || isempty(pages)
    pages = 1:size(imgPlusOffsetBgUint16, 3);
end

if ~exist('retFolder', 'var') || isempty(retFolder)
    retFolder = pwd;
end


if ~exist('tag', 'var') || isempty(tag)
    tag = 'imgAve';
end

if ~exist('Nbg', 'var') || isempty(Nbg)
    Nbg = 1000;
end


%% subtract offset, convert to photon units, and only do it for selected pages
imgPlusOffsetBgUint16 = imgPlusOffsetBgUint16(:,:,pages);
imgPlusBg = bsxfun(@minus, double(imgPlusOffsetBgUint16), double(cam.camOffSet));
% covert to photon units
imgPlusBg = imgPlusBg/cam.gain;

%% Analyze the NTile tiles  (9/25 etc): with the tiles.
mask = tiles.maskInnerRect;
maskPos = tiles.innerRectPosXYWHs;
% total photon of img+bg in each tile-mask
% total/mean photon of bg in each tile-mask
[Ny, Nx, Nz] = size(imgPlusBg);   %page number
T = size(tiles.centroidXYs, 2); %tile number
tilePhoton.sum_imgPlusBg = zeros(T, Nz);
for nz = 1:Nz
    for t = 1:T
        pos = maskPos(:,t); % [xMin, yMin, width, height]
        Icp = imgPlusBg(pos(2):pos(2) + pos(4), pos(1):pos(1) + pos(3), nz);
        tilePhoton.sum_imgPlusBg(t, nz) = sum(Icp(:));
    end
end

%% Analyze the NTile (9/25 etc) tiles: with the tiles.


% total photon of img+bg in each tile-mask
% total/mean photon of bg in each tile-mask
%%
pagesAllTile = repmat(pages, [T, 1]);
%%
gap = 1; idx = 1:gap:T;  legend1toT = sprintfc('%d', 1:T);% only show 1:gap:T tiles to avoid clutter
%% Show the total number of photons on each tile
figure(91); clf; hold on;
%plot(pagesAllTile(idx, :)', tilePhoton.sum_imgPlusBg(idx, :)'); title('photons for each tile'); grid on; grid minor;
lineColors = linspecer(numel(idx)); % ''qualitative' or 'sequential'
for i=1:numel(idx); plot(pages', tilePhoton.sum_imgPlusBg(idx(i), :)', 'color', lineColors(i,:)); end
title('photons for each tile'); grid on; grid minor;
legend(legend1toT(idx));

figure(191); clf; hold on;
for i=1:numel(idx); subplot(5,5, i); plot(pages', tilePhoton.sum_imgPlusBg(idx(i), :)'- tilePhoton.sum_imgPlusBg(idx(1), :)'); title(num2str(i)); end
title('photons for each tile'); grid on; grid minor;

%%
figure(92); clf;
subplot(121); plot(pages', sum(tilePhoton.sum_imgPlusBg,1)');  title('photons inside all tiles'); grid on; grid minor;
subplot(122); plot(pages', squeeze(sum(sum(imgPlusBg, 1),2))');  title('photons inside and outside tiles'); grid on; grid minor;

%% show each tile intensity profile
figure(93); set(gcf, 'units','normalized','outerposition',[0 0 1 1]); i=0;
for nz = 1:20:Nz %1:Nz
    for t = 1:12:T %t = 1:T        
        pos = maskPos(:,t); % [xMin, yMin, width, height]
        Icp = imgPlusBg(pos(2):pos(2) + pos(4), pos(1):pos(1) + pos(3), nz);                
        i = i+1;  if i>20; break; end        
        subplot(4, 5, i); sz= size(Icp);  plot(sum(Icp,1)/sz(1), 'rx-'); hold on; plot(sum(Icp,2)/sz(2), 'b+-'); legend('Photons along: x', 'Photons along: y'); grid on; grid minor; title(sprintf('nz=%d,t=%d', nz, t));
    end
end
printfig(fullfile(retFolder, sprintf('%s_VariesTilesDepths.png', tag)), [], 'cleanTitle', false);

%% show the center of each tile intensity profileget background of each tile
figure(194); set(gcf, 'units','normalized','outerposition',[0 0 1 1]); clf; i=0;
for nz = 1:20:Nz %1:Nz
    for t = 1:12:T %t = 1:T                     
        pos = maskPos(:,t); 
        Icp = imgPlusBg(pos(2):pos(2) + pos(4), pos(1):pos(1) + pos(3), nz); 
        % find the peak 
        sz= size(Icp);  
        [maxIx, xc]= max(sum(Icp,1)/sz(1));
        [maxIy, yc]= max(sum(Icp,2)/sz(2));
        %[~, idx] = max(Icp(:))
        yc = (1+sz(1))/2; xc = (1+sz(2))/2;
        i = i+1;  if i>20; break; end
        subplot(4, 5, i); imshow(Icp, [min(Icp(:)), max(Icp(:))]); hold on; plot(xc, yc, 'rx'); title(sprintf('nz=%d,t=%d', nz, t));
    end
end
%%
%%printfig(fullfile(retFolder, '%s_TileRectCenters.png'), [], 'cleanTitle', false);
printfig(fullfile(retFolder, sprintf('%s_TileCentersByPeakPhotonsAlongxy.png', tag)), [], 'cleanTitle', false);

%% show just simply use the point far away from the center of each tile to compute a background intensity for each tile
figure(95); set(gcf, 'units','normalized','outerposition',[0 0 1 1]); clf; i=0;
for nz = 1:20:Nz
    for t =  1:12:T 
        pos = maskPos(:,t); % [xMin, yMin, width, height]
        % mask and idx
        ySz = pos(4)+1; xSz = pos(3)+1;       
        [xx, yy] = meshgrid(1:xSz, 1:ySz);
        yc = (1+ySz)/2; xc = (1+xSz)/2; %[maxIx, xc]= max(sum(Icp,1)/sz(1)); [maxIy, yc]= max(sum(Icp,2)/sz(2));
        distMap = (xx-xc).^2 + (yy-yc).^2;
        [distSorted, idx] = sort(distMap(:), 'descend');                        
        Icp = imgPlusBg(pos(2):pos(2) + pos(4), pos(1):pos(1) + pos(3), nz);                
        i = i+1;  if i>20; break; end
        subplot(4, 5, i); plot(Icp(idx)); title(sprintf('nz=%d,t=%d, mean1k=%.2f', pages(nz), t, mean(Icp(idx(1:Nbg))))); %ylim([-1, 10]);  xlim([1, Nbg]);
    end
end
%%
%printfig(fullfile(retFolder, sprintf('%s_TileIntensityFromCenter_zoomed2.png', tag)), [], 'cleanTitle', false);
printfig(fullfile(retFolder, sprintf('%s_TileIntensityFromCenter.png', tag)), [], 'cleanTitle', false);


%% above shows background is trustable!
tilePhoton.bg = zeros(T, Nz);
tilePhoton.bgStd = zeros(T, Nz);
for nz = 1:Nz
    for t =  1:T 
        pos = maskPos(:,t); % [xMin, yMin, width, height]
        % mask and idx
        ySz = pos(4)+1; xSz = pos(3)+1;       
        [xx, yy] = meshgrid(1:xSz, 1:ySz);
        yc = (1+ySz)/2; xc = (1+xSz)/2; %[maxIx, xc]= max(sum(Icp,1)/sz(1)); [maxIy, yc]= max(sum(Icp,2)/sz(2));
        distMap = (xx-xc).^2 + (yy-yc).^2;
        [distSorted, idx] = sort(distMap(:), 'descend');                        
        %
        Icp = imgPlusBg(pos(2):pos(2) + pos(4), pos(1):pos(1) + pos(3), nz);  
        tilePhoton.bg(t, nz) = mean(Icp(idx(1:Nbg)));
        tilePhoton.bgStd(t, nz) = std(Icp(idx(1:Nbg)));
    end
end
%% remove the background
bg = zeros(size(imgPlusBg), 'like', imgPlusBg);
img = zeros(size(imgPlusBg), 'like', imgPlusBg);
for nz = 1:Nz
    for t = 1:T
        pos = maskPos(:,t); % [xMin, yMin, width, height]
        Icp = imgPlusBg(pos(2):pos(2) + pos(4), pos(1):pos(1) + pos(3), nz);  
        img(pos(2):pos(2) + pos(4), pos(1):pos(1) + pos(3), nz) = Icp - tilePhoton.bg(t, nz);
        bg(pos(2):pos(2) + pos(4), pos(1):pos(1) + pos(3), nz) = tilePhoton.bg(t, nz);
    end
end


%% Show the total number of photons on each tile
tilePhoton.sum_img = zeros(T, Nz);
tilePhoton.sum_imgNonNeg = zeros(T, Nz);
for nz = 1:Nz
    for t = 1:T
        pos = maskPos(:,t); % [xMin, yMin, width, height]
        Icp = img(pos(2):pos(2) + pos(4), pos(1):pos(1) + pos(3), nz);
        tilePhoton.sum_img(t, nz) = sum(Icp(:));
        tilePhoton.sum_imgNonNeg(t, nz) = sum(Icp(Icp>0));
    end
end

gap = 1; idx = 1:gap:T;  legend1toT = sprintfc('%d', 1:T);% only show 1:gap:T tiles to avoid clutter
figure(291); clf; hold on;
%plot(pagesAllTile(idx, :)', tilePhoton.sum_img(idx, :)'); title('photons for each tile'); grid on; grid minor;
lineColors = linspecer(numel(idx)); % ''qualitative' or 'sequential'
for i=1:numel(idx); plot(pages', tilePhoton.sum_img(idx(i), :)', 'color', lineColors(i,:)); end
title('obj photons for each tile'); grid on; grid minor;
legend(legend1toT(idx));

% figure(391); clf; hold on;
% for i=1:numel(idx); subplot(5,5, i); plot(pages', tilePhoton.sum_img(idx(i), :)'- tilePhoton.sum_img(idx(1), :)'); title(num2str(i)); end
% title('obj photons for each tile'); grid on; grid minor;

%% show the background

gap = 1; idx = 1:gap:T;  legend1toT = sprintfc('%d', 1:T);% only show 1:gap:T tiles to avoid clutter
figure(1291); clf; hold on;
%plot(pagesAllTile(idx, :)', tilePhoton.bg(idx, :)'); title('photons for each tile'); grid on; grid minor;
lineColors = linspecer(numel(idx)); % ''qualitative' or 'sequential'
for i=1:numel(idx); plot(pages', tilePhoton.bg(idx(i), :)', 'color', lineColors(i,:));  grid on; grid minor; end
title('background photons per pixel for each tile');
legend(legend1toT(idx));
%%
figure(1391); clf; hold on;
%for i=1:numel(idx); subplot(5,5, i); plot(pages', tilePhoton.bg(idx(i), :)' ./ tilePhoton.bg(idx(1), :)'); title(num2str(i)); end
for i=1:numel(idx); subplot(5,5, i); plot(pages', tilePhoton.bg(idx(i), :)'); title(num2str(i));  grid on; grid minor; end
%%


end

