function [xOut, flag] = extractContent(x, precision)
%extract content (scalar,1D,2D,3D matrix etc) from .mat/.tif/.tiff file or a matlab matrix (including empty matrix), may also cast to a certian precison/class
% Input:
%      x:          an array or  .mat/.tif/.tiff exsited file with full path.
%      precision:   opitional, specify xOut class 'uint8', 'uint16', 'single', 'double' etc.
% Output:
%      xOut:        extracted varialble
%      flag:       1 numeric input, 2 for .mat file, 3 for .tif/.tiff image file
if isempty(x) || isnumeric(x) % isnumeric() includes isempty(), use additional isempty() just for clarifying
    % (1) When x is numeric array: integer and float arrays (including []) etc, just pass the value to xOut.
    xOut = x;
    flag = 1;
elseif ischar(x) && exist(x, 'file')
    [~,~, ext] = fileparts(x);
    switch lower(ext)
        case '.mat'
            % (2) Load when x is a .mat file, load
            xOut  = load(x);
            fieldStrs = fieldnames(xOut);
            assert(numel(fieldStrs)==1, sprintf('The %s has stores %d > 1 varialbes, not sure which one to load!', x, numel(fieldStrs)));
            xOut = xOut.(fieldStrs{1});
            flag = 2;
        case {'.tif', '.tiff'}
            % (3) Read a image, when x is a tiff file
            xOut =  tif.read(x);
            flag = 3;
        otherwise
            error('Unkown psf file extention %s: ', ext);
    end
else
    error('Canot pass the input. It must be an array or  .mat/.tif/.tiff exsited file!');
end
if exist('precision', 'var') && ~isempty(precision)
    xOut = cast(xOut, precision);
end
end