function linkAxesXYZLimColorView(hf)
%set the same xyz limit/view/cameraPosition for all axes (such as created by subplot) in a figure
% Input
%   hf: handle of a figure, we will use current figure if hf is not specified
% Xiang Huang: xianghuang@gmail.com
% 2017.03.28: added to link the colormap 'Clim', 'CMode'

if ~exist('hf', 'var') || isempty(hf)
    hf = gcf;
end

ax = findobj(hf, 'type', 'axes');
%ax = hf.Children(arrayfun(@(x) isa(x, 'matlab.graphics.axis.Axes'), hf.Children)); % same as above, fin a figure could have children of (1) axes (2) UI objects, only select axes

% Find the axis limit that covers all axes
%XLimAll: N*2 array, kth row has [xmin, xmax] for kth axes. It stacks ax(k).XLim which is a 1*2 array
xLimAll = cat(1, ax(:).XLim); % Compare with  cat(2, ax(:).XLim), i.e., [ax(:).XLim], which is 1*2N array
yLimAll = cat(1, ax(:).YLim);
zLimAll = cat(1, ax(:).ZLim);

xLim = [min(xLimAll(:,1)), max(xLimAll(:,2))];
yLim = [min(yLimAll(:,1)), max(yLimAll(:,2))];
zLim = [min(zLimAll(:,1)), max(zLimAll(:,2))];

% Method I: linkprop, zoom in one axes will also automatically zoom in others (actually didn't work that way)h
% Link views so that rotation of view will remain in sync across both axes
hLink = linkprop(ax, {'XLim', 'YLim', 'ZLim', 'Color', 'View', 'CameraViewAngle', 'CLim', 'CMode'}); % can't add caxis?
setappdata(hf, 'graphfics_linkaxes', hLink);
ax(1).XLim = xLim;
ax(1).YLim = yLim;
ax(1).ZLim = zLim;

% 2017.02.15_added set caxis the same as the left most sub figure
caxis0 = caxis(ax(end)); % the left most ax;
for n=1:numel(ax)
    caxis(ax(n), caxis0);
end

end
% Other Method II: simpler choice, but can't zoom in same time
% axis(ax, [XLim, YLim, ZLim]);