%MFIPM_DEMO_NonNeg_Xiang Demonstrates the usage of the Matrix-free IPM solver.
%2017.09.10_Xiang modified from MFIPM_DEMO with constraints x>=0
% Comparison of two examples, with randn and rand nonnegative data, for the later nonnegative data, we also used the modified
% algorithm
% 1).randn(nMat, t)     1).rand(nMat,t)     1).rand(nMat,t).XiangModifiedNonNegAlgorithm
% (1915, 12)            (1579,10)           (3355, 35)      vs (6091, 61) with diagonal as preconditioned matrix
% (301, 1.2)            (253, 0.94)         (233, 1.2)
%  Two simple examples are shown.
%    1) One dimensional real Compressed Sensing problem with implicitily
%       stored Gaussian sensing matrix.
%    2) One dimensional real Compressed Sensing problem using an 1-d
%       partial DCT operator as a sensing matrix in the form of an A_operator.
%       ** Requires Signal Processing Toolbox **
%
% Copyright (c) 2013.  Kimon Fountoulakis, Jacek Gondzio and Pavel Zhlobich.

% ------------------------------------------------------
% Example 1, implicitily stored Gaussian sensing matrix.
% ------------------------------------------------------
fprintf('Example 1, Gaussian sensing matrix \n \n');

% Fix the seed.
randn('state',200);
rand('state',200);

% Number of measurements.
m = 2^10;

% Size of the signal.
n = 2^12;

% Number of nonzeros in the sparsest representation.
k = 51;

% Create the sensing matrix. Here we assume that the sparsity matrix
% is the identity.
A = randn(m,n);

% Create artificially an exact sparse representation.
p     = randperm(n);
x_opt = zeros(n,1); 

%2017.09.10_change from randn to rand for nonnegative x_opt
%x_opt(p(1:k)) = 2*randn(k,1);
x_opt(p(1:k)) = 2*rand(k,1);

% Calculate the measurements.
b = A*x_opt;

% Set the regularization parameter.
tau = 1.0e-3;

% Options.
opts.tol       = 1.0e-10;
opts.maxiters  = 100;
opts.tolpcg    = 1.0e-2;
opts.mxiterpcg = 200;
opts.verbose   = 2;

% Call the Matrix-free IPM solver.
[x, out] = mfipm(n, A, b, tau, opts);
[x, out] = mfipm_NonNeg_XH(n, A, b, tau, opts);

% Print info.
fprintf(['\n||x-x_opt||/||x_opt|| = %1.3e, Iters = %3d,'               ,...
         ' MatVecs = %5d \n \n'], norm(x-x_opt)/norm(x_opt), out.iters ,...
         out.nMat);

% ---------------------------------------------------------------
% Example 2, 1-d DCT sensing matrix in the form of an A_operator.
% ---------------------------------------------------------------
fprintf('Example 2, partial DCT matrix, ** Requires Signal Processing Toolbox ** \n \n');

% Partial 1-d discrete cosine transform. Here we assume that the sparsity
% matrix is the identity.
picks = randperm(n);
A = A_operator(@(x) pdct(x,1,n,sort(picks(1:m))),...
               @(x) pdct(x,2,n,sort(picks(1:m)))); 

% Create artificially an exact sparse representation.
p     = randperm(n);
x_opt = zeros(n,1); 

%2017.09.10_change from randn to rand for nonnegative x_opt
%x_opt(p(1:k)) = 2*randn(k,1);
x_opt(p(1:k)) = 2*rand(k,1);

% Calculate the measurements.
b = A*x_opt;

tau = 1.0e-5; % regularization parameter.

% Options.
opts.tol       = 1.0e-10;
opts.maxiters  = 100;
opts.tolpcg    = 1.0e-2;
opts.mxiterpcg = 200;
opts.verbose   = 2;

% Call the Matrix-free IPM solver.
[x out] = mfipm(n, A, b, tau, opts);
difference(x_opt, x)
[x out] = mfipm_NonNeg_XH(n, A, b, tau, opts); difference(x_opt, x)

% Print info.
fprintf(['\n||x-x_opt||/||x_opt|| = %1.3e, Iters = %3d,'               ,...
         ' MatVecs = %5d \n \n'], norm(x-x_opt)/norm(x_opt), out.iters ,...
         out.nMat);

clear;
