%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%NOTE: We set "isUseGPU = 1" by default for large system to use GPU for FFT (used in convolution). 
% Please set "isUseGPU = 0" if you don't have GPU or don't want to use GPU.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

addpath mfipm Skeleton3D

%% 1. Generate System:  Object and PSF
clear;
isDebug = 0; %0/1/2.    0 for real system, 1 for small size debug, 2 for medium size debug
isUseGPU = (isDebug ~=1);
smallWorldsPath = pwd; %fullfile('..', '..', '..',  'smallWorlds');

if isDebug
    if isDebug == 1
        tSz = [3, 3];
        psfSz = [3*tSz, 3];  % 11, or change to 7 to save computation
        fSz = psfSz; %   %fSz = [50, 50, psfSz(3)];
    elseif isDebug == 2
        tSz = [3, 3];        
        psfSz = [70*tSz, 90];  % 7/11/70, change to 7 for fast verification
        %fSz = [50, 50, psfSz(3)]; %fSz = [7, 7, psfSz(3)]; %
        fSz = psfSz; %
    else
        error('unkonwn debug opition isDebug = %d', isDebug);
    end
    %1a. Generate psf  two choices, real psf and random one
    [psf, tSz]  = simulatePSF(psfSz, tSz, false);
    rng(0, 'twister'); psf = rand(psfSz); psf = bsxfun(@rdivide, psf, sum(sum(psf,1),2));    
else    
    Ny = 512; Nx = Ny; Nz0 = 128;
    idxC = 128+(1:256); %center 256 portion of a 512 size
    idxZ = 1:8:Nz0; %idxZ = 1:gap:Nz;
    Nz = numel(idxZ);

    dataPath = pwd; %fullfile(smallWorldsPath, 'dataEPFL', 'microtubules');    
    psf0 = tif.read(fullfile(dataPath, 'psf.tif')); 
    % 256x512x128 --> crop center portion 256x256x128 --> pad to 512x512x128    
    psf = zeros(Ny, Nx, Nz, 'like', psf0);
    %psf(idxC, idxC, :) = psf0(:, idxC, idxZ);  
    rng(0, 'twister'); psf(idxC, idxC, :) = rand(numel(idxC), numel(idxC), Nz);  
    psf = psf/max(psf(:));
    
    
    fGT0 = tif.read(fullfile(dataPath, 'reference.tif'));  
    % 256x512x128 --> crop left portion 256x256x128 --> pad to 512x512x128        
    fGT = zeros(Ny, Nx, Nz, 'like', fGT0);
    fGT(idxC, idxC, :) = fGT0(:, 1:256, idxZ); 
    fGT = fGT/max(fGT(:));
    skel = Skeleton3D((fGT>0));
    fGT = cast(skel, class(psf)); fGT = fGT/max(fGT(:));
    
    psfSz = size(psf);    
    fSz = size(fGT);
end


% 1b. Generate 3D object fGT if its not already set
if ~exist('fGT', 'var') % if 1
    fc = ceil((fSz+1)/2);
    fGT = zeros(fSz, 'like', psf); 
    %fGT(fc(1), fc(2), fc(3)) = 1;
    fGT(fc(1), :, fc(3)) = 1;
end
% 1c. Specify observation image g size
gSz = psfSz(1:2);
% 1d. voxSz
voxSz = [1, 1, 1];
%% 2. Generate mfm model
mfmParas = {'PRECISION', 'double', 'CONVOLUTION_BOUNDARY', 'circular',  'verbose', 0,  'isUseGPU', isUseGPU};
mfm = MFM(psf, [], 'fGT', fGT, 'fSz', fSz, 'voxSz', voxSz, 'psfBgFile', 0, mfmParas{:}); %, 'gFrameNo', 1, 'verbose', 0)
g = mfm.AFun(mfm.fGT); % Exact forward model, no psf noise, no sensored g noise
mfm.setG(g); mfm.fGT = mfm.fGT/mfm.gNorm; % mfm.setG(g) set mfm.g = g/gNorm; so remember to also normalize fGT by gNorm
clear psf fGT g; % avoid confusing because of scale, always use mfm.psf/mfm.fGT/mfm.g
%% Hack to generate full AMatrix  to test for small cases when debug
isUseFullMatrixA = (isDebug==1);
if isUseFullMatrixA
    AMatrix = calcSensingMatrixViaPSF_CircConv(mfm.psf); % TODO-remove: HACK, full matrix for test
end

%% first solve noiseless
regWt = 1e-6;
tolA = 1e-8;
%solTypeAll = {'IPBoyd'}; preconditionerAll = {'diagATA'}; % Boyd's primary IP method, always use preconditioner 'diagATA', worse preformance than IPPDGond, not used for comparison in ICIP.
%solTypeAll = {'IPPDGond'}; preconditionerAll = {'identityATA'}; % Gondzio's feasible primer dual IP method, always uses preconditioner 'identityATA', so may no need to specify
%solTypeAll = {'IPPD'}; preconditioner = {'lowRankATA'};  % XH's primer dual IP method, could be feasible or infeasible  chooose from {'', 'zeroATA', 'identityATA', 'diagATA',  'lowRankATA'/{'lowRankATA', k}}; 
% 0:  '' or     []:     no preconditionr
% 1.  'zeroATA':    approximate  A'*A as a all zero matrix
% 2:  'identityATA':  approximate A'*A as a scaled identity matirx rho*ATA
% 3:  'diagATA':     approximate A'*A as diagonal matrix
% 4:  {'lowRankATA', k}:   approximate A'*A as rank k (currently rank 1) matrix lambda*v*v' use its largest eigenvalue lambda with corresponding eigenvector v%     
%solTypes = {'fista'}; % our implementation of FISTA, which seems to be worse than TwIST, not used for comparison in ICIP.
%solTypes = {'twist'}; % TwIST with non-negative thresholding after each iteration, which is faster and perform better than original TwIST without non-negative thresholding 
solTypeAll = {'IPPD', 'IPPDGond', 'twist'};
preconditionerAll = {'lowRankATA', 'identityATA', ''};

% 2018.02.06_test to compare the mfm.Vk,  2018.02.01_added rank-1 approximation of AMatrix. currently just do SVD, in future, do FFT
if exist('AMatrix', 'var') && ~isempty(AMatrix)
    [U0, S0, V0] = svd(AMatrix);
    k = 1; % why when k>1 increasing k needs more and more pcg iterations, k=2/3/4 is worse than k=1 for mainMFMTest() with isDebug = 1;
    Vk = V0(:, 1:k)*S0(1:k,1:k);
end

NSol = numel(solTypeAll);
solAll = cell(1, NSol);
timeAll = zeros(1, NSol);
self = mfm; normalizer = sqrt(sum(self.psf(:).^2)/numel(self.psf)*prod(self.fSz));
for nSolType = 1:numel(solTypeAll)
    solType = solTypeAll{nSolType};
    preconditioner = preconditionerAll{nSolType};
    tStart = tic;
    
    if strcmpi(solType, 'twist')        
        sampleInterval = [100, 200, 500, 1000, 2000, 5000, 10000, 20000];
        maxIterA  = 20000; %20000
    elseif strcmpi(solType, 'IPPD')
        sampleInterval = [10:50:150]; %sampleInterval = [10:10:100, 120:20:150];
        maxIterA = 150; 
    elseif strcmpi(solType, 'IPPDGond')
        sampleInterval = [10:50:200]; %sampleInterval = [10:10:100, 120:20:200];
        maxIterA = 200; 
    end
 
    %if strcmpi(solType, 'ip'); muCGAll = [2, 5,10]; end
    muCGAll = 2;
    for muCG = muCGAll
        solOpts = {'solType', solType, 'regFun', 'L1', 'isNonNegative', 1, 'maxIterA', maxIterA, 'regWt', regWt*normalizer^2, 'tolA', tolA, 'preconditioner', preconditioner, 'verbose', 1, 'isDebias', false, 'Lipschitz0', 5e3, 'muCG', muCG, 'sampleInterval', sampleInterval};
        if exist('isUseFullMatrixA', 'var') && isUseFullMatrixA && exist('AMatrix', 'var')      
            sol = SOLVER(mfm.g, AMatrix, 'xSz', mfm.fSz, 'xGT', mfm.fGT, 'diagATA', mfm.diagATA, 'Vk', mfm.Vk, solOpts{:}); %{b, AFun, 'xSz', xSz, 'name1', value1, 'name2', value2, ...}.        
        else
            sol = SOLVER(mfm, solOpts{:}); %
        end
        sol.solve();    
        extraPrefix = sprintf('%s_maxIterA=%d', solType, maxIterA); %'bgRemoveBoth_'  'bgRemoveMFM_' '', '
        sol.showSaveResult(mfm, extraPrefix);
        fprintf('muCG=%11.4e, totalItersCG=%d.\n', muCG, sum(sol.itersCG));
    end
end

