% more heavier lines especially for figur 1
%% specify data location
%smallWorldsPath = fullfile('..', '..', '..',  'smallworlds_SVN');
%folder =  fullfile(smallWorldsPath, 'Code', 'MFM', 'results');
folder = fullfile(pwd, 'results');


%ippdXHMat   = fullfile(folder, 'IPPD_maxIterA=150NONE_f001_20180210_0705_901s_solType=IPPD_regWt=2.0e-11_nVoxel=30380_nVoxelGT=4992_psnrSparse=45.75dB.mat');
ippdXHMat   = fullfile(folder, 'IPPD_maxIterA=150NONE_f001_20180210_0705_901s_solType=IPPD_regWt=2.0e-11_nVoxel=30380_nVoxelGT=4992_psnrSparse=45.75dB_rmHist.mat'); % retIppdXH.solPara.sampledHist =[]; % above data is large original takes up to 400MB, remove the sampledHist 
ippdGondMat = fullfile(folder, 'IPPDGond_maxIterA=200NONE_f001_20180210_0848_1773s_solType=IPPDGond_regWt=2.0e-11_nVoxel=608417_nVoxelGT=4992_psnrSparse=0.49dB.mat');
twistMat    =   fullfile(folder, 'twist_maxIterA=20000NONE_f001_20180210_0741_2392s_solType=twist_regWt=2.0e-11_nVoxel=100687_nVoxelGT=4992_psnrSparse=3.07dB.mat');

%% load data:  require: solPara.cpuTimes, solPara.reconRelativeRMSE, solPara.objFunVals
retIppdXH = load(ippdXHMat);
retIppdGond = load(ippdGondMat);
retTwist = load(twistMat);
retAll = {retIppdXH, retIppdGond, retTwist};
solParaAll = {retIppdXH.solPara, retIppdGond.solPara, retTwist.solPara};

fGT = retIppdXH.mfmPara.fGT; 
fGTMax = max(fGT(:));
fGTMeanSquareSparse = sum(fGT(:).^2) / sum(fGT(:)>10*eps*max(fGT(:)));
fGTMeanSquare = sum(fGT(:).^2) / numel(fGT);
psnrSparseFun = @(relativeRMSE)  10*log10(fGTMax.^2./ (relativeRMSE.^2 * fGTMeanSquareSparse));
psnrFun = @(relativeRMSE) 10*log10(fGTMax.^2./ (relativeRMSE.^2 * fGTMeanSquare));


%%
legendNames = {'IPM-ours', 'IPM-FOU',  'TwIST'};
%lineStyles = {'r-', 'b-.', 'k--'};
lineStyles = {'r-', 'b-.', 'c'};
myfontSz = 16;
%% show PSNR vs time
figure(1),clf;
for n = 1:numel(solParaAll)
    solPara = solParaAll{n};
    plot(solPara.cpuTimes, psnrFun(solPara.reconRelativeRMSE), lineStyles{n}, 'LineWidth', 2);
    %semilogy(solPara.cpuTimes, psnrFun(solPara.reconRelativeRMSE), lineStyles{n});
    hold on;
end
legend(legendNames{:});
title('PSNR:  ippdXH vs ippdGond vs vs twist ');
xlabel('Computation time (seconds)');
ylabel('PSNR (dB)');
grid on; grid minor;
curTime = datestr(now, 'yyyymmdd_HHMM');
prefix = 'XHvsGondvsTwist_PSNR_';
figName = fullfile(folder, [prefix, curTime '.png']);
ylim([-20, 80]);
printfig(figName, [1600, 1200], 'fontsize', myfontSz, 'cleanTitle', true)        

ylim([-20, 80]);
xlim([0, 20]);
hLegend=get(gca, 'legend'); set(hLegend, 'Location', 'SouthEast')
figZoomedName = fullfile(folder, [prefix, curTime '_ZoomedIn.png']);
printfig(figZoomedName, [1000, 1200], 'fontsize', myfontSz, 'cleanTitle', true)     


% show objective function value vs time
figure(9); clf;  %set(gca, 'YScale', 'log');  hold on;
for n = 1:numel(solParaAll)
    solPara = solParaAll{n};
    semilogy(solPara.cpuTimes, solPara.objFunVals, lineStyles{n}, 'LineWidth', 2); 
    hold on;
end

%hold on; semilogy(retIppdXH.solPara.cpuTimes, retIppdXH.solPara.dualFunVals, 'c');
legend(legendNames{:}); %legend(legendNames{:}, 'Location', 'BestOutside');
xlabel('computation time (seconds)');
ylabel('objective function value h(x)');
title('Objective function value versus time');
grid on; grid minor;

curTime = datestr(now, 'yyyymmdd_HHMM');
prefix = 'XHvsGondvsTwist_ObjFun_';
figName = fullfile(folder, [prefix, curTime '.png']);
ylim([10^(-8), 10^8])
printfig(figName, [1600, 1200], 'fontsize', myfontSz, 'cleanTitle', true)        

ylim([10^(-8), 10^8]);
xlim([0, 100]); %ylim([5*1e-5, 5*1e-4])
figZoomedName = fullfile(folder, [prefix, curTime '_ZoomedIn.png']);
printfig(figZoomedName, [1000, 1200], 'fontsize', myfontSz, 'cleanTitle', true)     


%%  show volumes
[Ny0, Nx0, Nz0] = size(fGT);
Ny = Ny0/2; Nx = Nx0/2; idx = 129:384;
voxSz = [100, 100, 800];
figSz = [1600, 1200];
fMax = fGTMax; %for n = 1:numel(solParaAll); fMax = max(fMax, max(solParaAll{n}.f(:))); end
projMax = max(max(sum(fGT,3))); for n = 1:numel(solParaAll); projMax = max(projMax, max(max(sum(solParaAll{n}.f,3)))); end
legendNamesPlusGT = {legendNames{:}, 'GT'};
prefix1 = 'volumeImg_';
prefix2 = 'projImg_';
curTime = datestr(now, 'yyyymmdd_HHMM');
for n = 1:(numel(solParaAll)+1)
    if n > numel(solParaAll)
        f = fGT(idx, idx, :);
    else
        f = solParaAll{n}.f(idx, idx,:);
    end
    figure(30+n); clf;
    show3dPointsWithProjections(f/fMax, voxSz, 1e-2, 'maxF', fMax, 'markersize', 2);       %show3dPointsWithProjections(f, voxSz, 1e-2, 'maxF', fMax);       
    figName = [prefix1, legendNamesPlusGT{n}, '_', curTime '.png'];
    title(figName);
    printfig(fullfile(folder, figName), [1600, 1200], 'fontsize', 10, 'cleanTitle', true)         
    figure(40+n);clf;
    %imagesc([1, Ny]*voxSz(1), [1, Nx]*voxSz(2), (sum(f,3)/projMax).^0.5); axis image;   caxis([0, 1]);    %imagesc([1, Ny]*voxSz(1), [1, Nx]*voxSz(2), sum(f,3)); axis image;   caxis([0, projMax]);    
    imgSpec = {'XData', [1, Ny]*voxSz(1),  'YData', [1, Nx]*voxSz(2), 'DisplayRange', [0, 1]};
    hImg1 = imshow((sum(f,3)/projMax).^0.5, imgSpec{:}); colormap('jet');     
    figName = [prefix2, legendNamesPlusGT{n}, '_', curTime '.png'];
    title(figName);
    printfig(fullfile(folder, figName), [1024, 1024]); %, 'fontsize', 10, 'cleanTitle', true)
end

%% Code only used to generate colorbar for illustration
%{
load('g.mat');
g = g(128+(1:256), 128+(1:256));
[Ny2, Nx2] = size(g);
figure(2); clf; imagesc( [1, Nx2]*voxSz(2), [1, Ny2]*voxSz(1), g); axis image; colormap('jet'); colorbar; xlabel('x (nm)'); ylabel('y (nm)')
printfig('g_jet.png', [1600, 1600], 'fontsize', 14);
%}