function [psf, tSz] = simulatePSF(psfSz, tSz, isShow)
%Generate a PSF of size psfSz, each z-slice has tSz subtiles, each with psftSz Pixels.
%% Pass input and Parameter Choices
if ~exist('tSz', 'var') || isempty(tSz)
    tSz = [3, 3];
end
if ~exist('psfSz', 'var') || isempty(psfSz)
    psfSz = [[81, 81].*tSz, 101]; 
end
if ~exist('isShow', 'var') || isempty(isShow)
    isShow = false;
end
psfSz(1:2) = round(psfSz(1:2) ./ tSz) .* tSz;
psftSz = psfSz(1:2) ./ tSz; 
%% 2. Simulate Gauss Blur, in future, simulate Fresnel Diffraction or real PSF
psf = zeros(psfSz);
y = linspace(0,1,psftSz(1));
x = linspace(0,1,psftSz(2)); 
muY = mean(y); 
muX = mean(x); 
[Y, X] = ndgrid(y, x);
centralTile = (1+tSz(2)*tSz(1))/2; % 
for cTile = 1:tSz(2)
    for rTile = 1:tSz(1)    
        % Generate for sub-tile with region [0,1]x[0,1]
        idxTile = rTile + (cTile-1)*tSz(1); % idxTile from 1 to tSz(2)*tSz(1)
        if prod(tSz)>1
            zShift = -(idxTile-centralTile)/(tSz(2)*tSz(1)-1);% multiple tiles, zShift from +0.5 to -0.5;
        else
            zShift = 0; % only one tile
        end
        z = linspace(0,1,psfSz(3)) + zShift;  % central tile has depth z which ranges from 0 to 1, other tiles with zShift offset
        muZ = mean(z);
        sigmaAll = abs(z-0.5) + 0.1; % sigma promotional to |z-0.5|, but adding 0.01 to avoid 0 sigma for central tile;
        psfSub = zeros([psftSz, psfSz(3)]);
        for iz = 1:numel(z)
            sigma = sigmaAll(iz);
            psfSubz = exp(- ((Y - muY).^2 + (X - muX).^2) / (2*sigma^2)) / (2*pi*sigma^2);
            psfSubz = psfSubz/sum(psfSubz(:));
            psfSub(:,:,iz) = psfSubz;
            %figure, imagesc(PSFz); colormap('gray'); axis image
        end
        psfSub = psfSub/max(psfSub(:));
        %sliceomatic(psfSub, x, y, z); axis ij;               
        psf((1:psftSz(1)) + (rTile-1)*psftSz(1),  (1:psftSz(2)) + (cTile-1)*psftSz(2), :) = psfSub;
    end
end
if isShow
    sliceNo = [1, floor((psfSz(3)+1)/2), psfSz(3)];
    figure, multAxes(@imagesc, {psf(:,:,sliceNo(1)), psf(:,:,sliceNo(2)), psf(:,:,sliceNo(3))});  
    multAxes(@axis, 'image'); multAxes(@colorbar); multAxes(@title, {['PSF: Top z slice: ' num2str(sliceNo(1))], ['Central z slice: ' num2str(sliceNo(2))], ['Bottom z slice: ' num2str(sliceNo(3))]});
    if psfSz(3)>1
        sliceomatic(psf); axis ij;  daspect([1 1 psfSz(3)/psfSz(1)]); xlim([1, psfSz(2)]); ylim([1, psfSz(1)]); zlim([1 psfSz(3)]);
    end
    %% show the psf blurred with a bead (just show with gaussian for illustration)
%     bead = fspecial3('ellipsoid', 4);
%     psfBlurred = convn(psf, bead,'same');    
%     figure, multAxes(@imagesc, {psfBlurred(:,:,1), psfBlurred(:,:,floor((psfSz(3)+1)/2)), psfBlurred(:,:,end)}); 
%     multAxes(@axis, 'image'); multAxes(@colorbar); multAxes(@title, {'PSF Blurred: Top z slice', 'Central z slice', 'Bottom z slice'});
%     sliceomatic(psfBlurred); axis ij;  daspect([1 1 psfSz(3)/psfSz(1)]); xlim([1, psfSz(2)]); ylim([1, psfSz(1)]); zlim([1 psfSz(3)]);
end


end

function h = fspecial3(type,siz)
%FSPECIAL3 Create predefined 3-D filters.
%   H = FSPECIAL3(TYPE,SIZE) creates a 3-dimensional filter H of the
%   specified type and size. Possible values for TYPE are:
%
%     'average'    averaging filter
%     'ellipsoid'  ellipsoidal averaging filter
%     'gaussian'   Gaussian lowpass filter
%     'laplacian'  Laplacian operator
%     'log'        Laplacian of Gaussian filter
%
%   The default SIZE is [5 5 5]. If SIZE is a scalar then H is a 3D cubic
%   filter of dimension SIZE^3.
%
%   H = FSPECIAL3('average',SIZE) returns an averaging filter H of size
%   SIZE. SIZE can be a 3-element vector specifying the dimensions in
%   H or a scalar, in which case H is a cubic array.
%
%   H = FSPECIAL3('ellipsoid',SIZE) returns an ellipsoidal averaging filter.
%
%   H = FSPECIAL3('gaussian',SIZE) returns a centered Gaussian lowpass
%   filter of size SIZE with standard deviations defined as
%   SIZE/(4*sqrt(2*log(2))) so that FWHM equals half filter size
%   (http://en.wikipedia.org/wiki/FWHM). Such a FWHM-dependent standard
%   deviation yields a congruous Gaussian shape (what should be expected
%   for a Gaussian filter!).
%
%   H = FSPECIAL3('laplacian') returns a 3-by-3-by-3 filter approximating
%   the shape of the three-dimensional Laplacian operator. REMARK: the
%   shape of the Laplacian cannot be adjusted. An infinite number of 3D
%   Laplacian could be defined. If you know any simple formulation allowing
%   one to control the shape, please contact me.
%
%   H = FSPECIAL3('log',SIZE) returns a rotationally symmetric Laplacian of
%   Gaussian filter of size SIZE with standard deviation defined as
%   SIZE/(4*sqrt(2*log(2))).
%
%   Class Support
%   -------------
%   H is of class double.
%
%   Example
%   -------
%      I = single(rand(80,40,20));
%      h = fspecial3('gaussian',[9 5 3]); 
%      Inew = imfilter(I,h,'replicate');
%       
%   See also IMFILTER, FSPECIAL.
%
%   -- Damien Garcia -- 2007/08

error(nargchk(1,2,nargin))
type = lower(type);

if nargin==1
        siz = 5;
end

if numel(siz)==1
    siz = round(repmat(siz,1,3));
elseif numel(siz)~=3
    error('Number of elements in SIZ must be 1 or 3')
else
    siz = round(siz(:)');
end

switch type
    
    case 'average'
        h = ones(siz)/prod(siz);
        
    case 'gaussian'
        sig = siz/(4*sqrt(2*log(2)));
        siz   = (siz-1)/2;
        [x,y,z] = ndgrid(-siz(1):siz(1),-siz(2):siz(2),-siz(3):siz(3));
        h = exp(-(x.*x/2/sig(1)^2 + y.*y/2/sig(2)^2 + z.*z/2/sig(3)^2));
        h = h/sum(h(:));
        
    case 'ellipsoid'
        R = siz/2;
        R(R==0) = 1;
        h = ones(siz);
        siz = (siz-1)/2;
        [x,y,z] = ndgrid(-siz(1):siz(1),-siz(2):siz(2),-siz(3):siz(3));
        I = (x.*x/R(1)^2+y.*y/R(2)^2+z.*z/R(3)^2)>1;
        h(I) = 0;
        h = h/sum(h(:));
        
    case 'laplacian'
        h = zeros(3,3,3);
        h(:,:,1) = [0 3 0;3 10 3;0 3 0];
        h(:,:,3) = h(:,:,1);
        h(:,:,2) = [3 10 3;10 -96 10;3 10 3];
        
    case 'log'
        sig = siz/(4*sqrt(2*log(2)));
        siz   = (siz-1)/2;
        [x,y,z] = ndgrid(-siz(1):siz(1),-siz(2):siz(2),-siz(3):siz(3));
        h = exp(-(x.*x/2/sig(1)^2 + y.*y/2/sig(2)^2 + z.*z/2/sig(3)^2));
        h = h/sum(h(:));
        arg = (x.*x/sig(1)^4 + y.*y/sig(2)^4 + z.*z/sig(3)^4 - ...
            (1/sig(1)^2 + 1/sig(2)^2 + 1/sig(3)^2));
        h = arg.*h;
        h = h-sum(h(:))/prod(2*siz+1);
        
    otherwise
        error('Unknown filter type.')
end
end