function [x, objs, cpuTimes, SNRs, sampledHist, regFunVals, duals, ts, itersCG, out] = solveIPPD(varargin)
%function [x, objFunVals, cpuTimes, SNRs] = solveIP(b, A, regWt, varargin)
% infeasible interior point with preconditioned conjugate gradient method
%refer to twist for parameter explainations, will copy the explaination here once all set
%           Lipschitz0, ...  % FISTA Only Parameters
%       smaller means better convergence, but can't be smaller than xx.
% test for number of required parametres
% the result for kth iteration is stored in objFunVals(k+1), cpuTimes(k+1) etc., for k start with 0.
%assert((nargin-numel(varargin)) == 3, 'Wrong number of required parameters');
% 2017.09.24 feasible IPPD bad! found x.*s not balanced, some are very small

isDebugWithFullAMatrix = false;

%% Parse Inputs
opts =  solver.parseInput(varargin{:});
cellfun(@(x) evalin('caller', sprintf('%s=opts.%s;', x, x)), fields(opts)); % exact each field as individual varibles


% vectorize
x0 = x0(:); xGT = xGT(:); b = b(:);
% prepare constants
N  = numel(x0);
M  = numel(b);
bNorm  = sqrt(b(:)'*b(:));
regWt_ATb = regWt - A'*b;  % objFunGrad = grad(objFun(x)) = A'Ax + regWt_ATb; NOTE: A'*b won't appear along in any where, always come with regWt - A'*b

% Objective function and dual function values
objFun = @(x, r) 0.5*(r'*r) + regWt*sum(abs(x));  % 2018.02.05_added abs(x) for future to extend from non-negative x to real/complex x.; % h(x),,  overide the more general objFun = @(x,resid,regWt,regFun)0.5*(resid'*resid)+regWt*regFun(x)
dualFun = @(Ax) - 0.5*(Ax'*Ax) + 0.5*bNorm^2;  % if s is dual feasible, then x'*s = objFun(x,r) - dualFun(Ax);

%%% ippd only parameters
optsIPPD.tolpcg    = 1.0e-2;
optsIPPD.mxiterpcg = 100;
optsIPPD.maxstepsz = 0.999; % not used
optsIPPD = chkOptargs(optsIPPD);


isDebugWithExistFun = false;
if isDebugWithExistFun
    % Call the Matrix-free IPM solver.
    [x, out] = mfipm_NonNeg_XH(prod(xSz), A, b(:), regWt, optsIPPD);
    x = reshape(x, xSz);
    return
end

%% Initalize Output Arrays: since we can't store objFunVals(0), we use objFunVals(k+1) to represent steps k = 0, 1, ..., maxIterA
tic;  % start the clock
sampledHist = [];
objs = zeros(1, maxIterA+1);
regFunVals  = zeros(1, maxIterA+1); % regulizer function values, such as abs(sum(x(:)) for L1-regulizer
duals = zeros(1, maxIterA+1); % dual objective values
ts = zeros(2, maxIterA+1); % history of sigma_a; sigma_c parameter
itersCG = zeros(1, maxIterA+1); % history of # of iterations in each CG/newton step
relResCG = zeros(1, maxIterA+1); % relative error of pcg solver
cpuTimes = zeros(1, maxIterA+1);
SNRs = zeros(1, maxIterA+1);

% Counters.
out.predictors  = 0;
out.correctors  = 0;
out.linsystems  = 0;
out.nMat        = 0;
out.totcputime  = 0;
out.nMat = out.nMat + 1;




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
k = 0;
%clear Vk % debug to use full Vk
if ~exist('Vk', 'var') && exist('AMatrix', 'var') && ~isempty(AMatrix)
    [U0, S0, V0] = svd(AMatrix);
    K = 1; % why when k>1 increasing k needs more and more pcg iterations, k=2/3/4 is worse than k=1 for mainMFMTest() with isDebug = 1;
    Vk = V0(:, 1:K)*S0(1:K,1:K);
end
% Gondiz's Intialization of x & s, pay attention
isUseGondizInitialization = 0; %1/0
if isUseGondizInitialization
    dInv = 1.0e-2*ones(N,1);
    HFun = @(dx) A'*(A*dx) + dInv.*dx; % function handle of (Dinv + A'A)*dx
    MInvFun = solver.getMInvFun(preconditioner, 1./dInv, M/N, diagATA, Vk); % function handle M\x for preconditioner preconditioner M
    [x, flag, relResCG(k+1), itersCG(k+1)] = pcgXH(HFun, regWt-A'*b, 1.0e-2, 200, MInvFun); %  
    x = abs(x) + bNorm;
    s = bNorm*ones(N,1);
    figure; multAxes(@plot, {x, s, s - (A'*(A*x) + regWt_ATb), x.*s}); multAxes(@title, {'x', 's', 'r_d', 'x.*s'});
else
    [x, s]  = calcInitial(A, b, regWt);
end
Ax = A*x; ATAx = A'*(Ax);
[duals(k+1), nu] = calcDual(b - Ax, ATAx + regWt_ATb, b, regWt); % dual for infeasible s (s>=0, but s ~= grad(h(x))
out.nMat = out.nMat + 3;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Print info.
if verbose
    fprintf(' %5s | %5s |  %5s   | %8s | %8s | %9s | %10s |%10s \n', ' Info ', 'Iters',...
        'rDGap', 'rDualInf','Iters PCG', 'RelRes PCG', 'Primal Step', 'Dual Step');
    fprintf('  -------------------------------------------------------------------------------------- \n');
end

cpuTimes(k+1) = toc;
regFunVals(k+1)  = regWt*sum(abs(x));  % 2018.02.05_added abs(x) for future to extend from non-negative x to real/complex x.; %; regFun(x);
objs(k+1) = objFun(x, b-Ax);
eta = objs(k+1) - duals(k+1); % duality gap
r_d  = s - (ATAx + regWt_ATb);
fprintf('xs = %.8e, norm(r_d) = %.8edB.\n', x'*s, norm(r_d));   
if ~isempty(xGT)
    [~, SNRs(k+1)] = psnr(x, xGT);
    rGT = b - A*xGT;
    objFunValGT = objFun(xGT, rGT);  % objFun(xGT, b-A*xGT, regWt, regFun);  % compute using the passed function
else
    objFunValGT = [];
end

solver.printAlgorithmBegin('algorithmName', mfilename, 'verbose', verbose, 'x', x, 'objFunVals', objs, 'regWt', regWt, 'regFun', regFun, 'xGT', xGT);

% default status set as finished all loops and reach maximum iteration, unless stop criteria meet to break the loops
out.status  = 0;
out.message = 'Maximum iterations reached';


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%tolCG = max(0.001, min(0.1, 1*eta/bNorm));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
sigma1 = 0.1;
sigma2 = 0.5;
sigma3 = 0.8; % change to 0.99?
alpha1 = 0.1;
alpha2 = 0.5;

%optsIPPD.mxiterpcg = 500; % debug
sigma1=0.01; sigma2=0.01; sigma3=0.99; alpha1=0.1; alpha2=0.5; % 0.01 and 0.99 are much faster
fprintf('sigma1 = %.2f, sigma2 = %.2f, sigma3 = %.2f, alpha1 = %.2f, alpha2 = %.2f.\n', sigma1, sigma2, sigma3, alpha1, alpha2); %sigma1 = 0.01; sigma2 = 0.5; sigma3 = 0.9;
xPrev = x;
sPrev = s;
for k = 1:maxIterA    
    %% Predictor
    % 1. Set sigma from previous step length.    
    marker = 'Pred.';
    out.predictors  = out.predictors + 1;    
    if (k == 1) || (min(alphaP, alphaD) > alpha2)
        sigma = sigma1; % Predictor: Compute aggressive affine scaling direction, because previous step is long step
    else
        sigma = sigma2; % Predictor: Compute moderate affine scaling direction, because previous step is not that long < 0.5.
    end 
    ts(1,k+1) = sigma; % history predictor sigmas   

    % 2. Solve Newton step (dx, ds) from (xPrev, sPrev, sigma, objFunGrad = ATAx + regWt_ATb)     
    MInvFun = solver.getMInvFun(preconditioner, xPrev./sPrev, M/N, diagATA, Vk); % function handle M\x for preconditioner M used in PCG
    [dxPred, dsPred, relResCG_curIter, itersCG_curIter] = calcNewtonStep(A, xPrev, sPrev, sigma, ATAx + regWt_ATb, optsIPPD.tolpcg, optsIPPD.mxiterpcg, MInvFun);
    relResCG(k+1) = relResCG(k+1) + relResCG_curIter;  itersCG(k+1) = itersCG(k+1)+ itersCG_curIter; 

    % 3. Compute Step Size to make satisfies dual feasiblility (KKT stationary) condition s = A'*Ax + regWt_ATb
    alphaP = solver.calcStepSize(xPrev, dxPred, optsIPPD.maxstepsz); % use seperate alphaP and alphaD saves 10%-20% computation time vs use same step size
    alphaD = solver.calcStepSize(sPrev, dsPred, optsIPPD.maxstepsz);
    %alphaP = min(alphaP, alphaD); alphaD = min(alphaP, alphaD); % % 2018.02.07_DEBUG_Bad
    
    % Update x/s
    x = xPrev + alphaP*dxPred; % x2 = x + alphaP*dx;
    s = sPrev + alphaD*dsPred; % s2 = s + alphaD*ds;
    
    % Print info.
    if verbose
            fprintf('  %5s | %3d   | %8s | %8s | %5d     |  %1.2e  |   %1.2e  |  %1.2e \n',...
                    marker, k, '', '', itersCG(k+1), relResCG(k+1), alphaP, alphaD);           
    end
    mu = x'*s/N; fprintf('sigma = %.1e, tolCG=%.1e, xTs = %.8e, min(x.*s)/(xTs/N) = %.3e,  norm(r_d) = %.8e.\n',sigma, optsIPPD.tolpcg, mu*N, min(x.*s)/mu, norm(r_d));

    out.nMat  = out.nMat + 2*itersCG(k+1);
    
    %% Corrector
	if min(alphaP, alphaD) < alpha1
        marker = 'Corr.';
        out.correctors  = out.correctors + 1;        
        sigma = sigma3; % Corrector: Compute centering direction, if needed.
        ts(2,k+1) = sigma; % history corrector sigmas
        
        
        % 2. Solve Newton step (dx, ds) from (x, s, sigma, objFunGrad = ATAx + regWt_ATb)     
        isTest = 1;
        if isTest
            MInvFun = solver.getMInvFun(preconditioner, xPrev./sPrev, M/N, diagATA, Vk); % function handle M\x for preconditioner M used in PCG
            [dxCorr, dsCorr, relResCG_curIter, itersCG_curIter] = calcNewtonStep(A, xPrev, sPrev, sigma, ATAx + regWt_ATb, optsIPPD.tolpcg, optsIPPD.mxiterpcg, MInvFun); % NoTe here we use the predictor updated x, not the xPrev
            relResCG(k+1) = relResCG(k+1) + relResCG_curIter;  itersCG(k+1) = itersCG(k+1)+ itersCG_curIter; 
            % 2.2 Combine the direction from predictor and corrector
            dx = dxCorr;
            ds = dsCorr;
        else            
            MInvFun = solver.getMInvFun(preconditioner, x./s, M/N, diagATA, Vk); % function handle M\x for preconditioner M used in PCG
            [dxCorr, dsCorr, relResCG_curIter, itersCG_curIter] = calcNewtonStep(A, x, s, sigma, ATAx + regWt_ATb, optsIPPD.tolpcg, optsIPPD.mxiterpcg, MInvFun); % NoTe here we use the predictor updated x, not the xPrev
            relResCG(k+1) = relResCG(k+1) + relResCG_curIter;  itersCG(k+1) = itersCG(k+1)+ itersCG_curIter; 
            % 2.2 Combine the direction from predictor and corrector
            dx = dxPred + dxCorr;
            ds = dsPred + dsCorr;
        
        end
        % 3. Compute Step Size to make satisfies dual feasiblility (KKT stationary) condition s = A'*Ax + regWt_ATb
        alphaP = solver.calcStepSize(xPrev, dx, optsIPPD.maxstepsz); % NOTE here we still use xPrev, not the predictior updated x
        alphaD = solver.calcStepSize(sPrev, ds, optsIPPD.maxstepsz);        
        %alphaP = min(alphaP, alphaD); alphaD = min(alphaP, alphaD); % % 2018.02.07_DEBUG_Bad

        % Update x/s
        x = xPrev + alphaP*dx; % x2 = x + alphaP*dx;
        s = sPrev + alphaD*ds; % s2 = s + alphaD*ds;

        % Print info.
        if verbose
                fprintf('  %5s | %3d   | %8s | %8s | %5d     |  %1.2e  |   %1.2e  |  %1.2e \n',...
                        marker, k, '', '', itersCG(k+1), relResCG(k+1), alphaP, alphaD);           
        end
        mu = x'*s/N; fprintf('sigma = %.1e, tolCG=%.1e, xTs = %.8e, min(x.*s)/(xTs/N) = %.3e,  norm(r_d) = %.8e.\n',sigma, optsIPPD.tolpcg, mu*N, min(x.*s)/mu, norm(r_d));
        
        out.nMat  = out.nMat + 2*itersCG(k+1);        
    end    
    
    %% Update new objective/dual function values, and Ax, ATAx for next step
    Ax = A*x; % Ax = Ax + alphaP*Adx;
    ATAx = A'*(Ax); % ATAx = ATAx + alphaD*ATAdx;   
    objs(k+1) = objFun(x, b-Ax);
    regFunVals(k+1)  = regWt*sum(abs(x));  % 2018.02.05_added abs(x) for future to extend from non-negative x to real/complex x.; %; regFun(x);
    %duals(k+1) = dualFun(Ax); % dual for feasible (x,s):  -0.5*x'*ATAx + 0.5*bNorm^2;
    [duals(k+1), ~] = calcDual(b - Ax, ATAx + regWt_ATb, b, regWt); % dual for infeasible s (s>=0, but s ~= grad(h(x))

    eta = objs(k+1) - duals(k+1); % should be similar to x'*s
    out.nMat  = out.nMat + 2;  % may comment out if use Ax = Ax + alphaPD*Adx; etc

    % Termination criteria.
    mu        = x'*s/N;
    rel_dgap  = mu/(1.0 + abs(objs(k+1))); % original, same as 1+abs(0.5*norm(A*x-b)^2+regWt*sum(x))

    r_d  = s - (ATAx + regWt_ATb);
   
    % Print info.
    if verbose
        fprintf('  %5s | %3d   | %1.2e | %1.2e | \n',  'final', k, rel_dgap, norm(r_d));        
    end   

    % store and print out results for current step
    cpuTimes(k+1) = toc;
    if ~isempty(xGT);  [~, SNRs(k+1)] = psnr(x, xGT); end
    sampledHist = solver.updateSampledHist(sampledHist, x, cpuTimes, k, sampleInterval); % Only keep x history every sampleInterval

    %criterion = eta; %  absolute obj error
    if (eta < 0 || objs(k+1)<0); warning('eta=%.e<0, objs(k+1)=%.e<0', eta, objs(k+1)); end
    %criterion = max(abs(eta) ./ abs([objs(k+1), duals(k+1)])); % relative obj error    
    criterion = abs(eta) ./ abs(objs(k+1)); % relative obj error is no more than this number, as objs > 0 and decreasing
    solver.printAlgorithmIterK('k', k, 'verbose', verbose, 'x', x, 'objFunVals', objs, 'criterionVsTol', criterion/tolA, 'regWt', regWt, 'regFun', regFun, 'xGT', xGT, 'dualFunVals', duals);    
        
%    dualIfFeasible = dualFun(Ax); eta2 = objs(k+1)-dualFun(Ax); criterion2 = max(abs(eta2) ./ abs([objs(k+1), dualIfFeasible])); % relative obj error
%    if (rel_dgap < optsIPPD.tol) && (criterion2<=tolA) && (k>minIterA) % (rel_dgap < optsIPPD.tol) && (criterion<=tolA) maybe duplicate
    if (criterion<=tolA*100) && norm(r_d)<tolA*100 && (k>minIterA) % since we computed the true dual, we only need  to make sure the objective function value is close to optimal
        fprintf('tolA=%.2e\n', tolA);
        out.status  = 3;
        out.message = sprintf('Converged #1: mu/(1.0+ |obj|) < tolA:  %.2e / (1 + |%.2e|) = %.2e < %.2e.\n', mu, objs(k+1), rel_dgap, tolA);
        out.message = sprintf('%sConverged #2: max(eta ./ |[obj, dual]|) < toA:   max(%.2e / |[%.8e, %.8e]| = %.2d < %.2e', out.message, eta, objs(k+1), duals(k+1), criterion, tolA);
        break;
    end
    
    % prepare for next step
    xPrev = x;
    sPrev = s;
end

fprintf('%s\n', out.message);
out.linsystems = out.linsystems + out.correctors + out.predictors;
out.totcputime = toc;

if verbose
    fprintf('\nTotal CPU time: %f \n', out.totcputime);
    fprintf('Iterations: %3d \n', k);
    fprintf('Predictors: %3d \n', out.predictors);
    fprintf('Correctors: %3d \n', out.correctors);
    fprintf('Total linear systems solved: %3d \n', out.linsystems);
    fprintf('Total PCG iterations: %4d  \n', sum(itersCG));
    fprintf('Total nMat: %5d \n', out.nMat);
end

x = reshape(x, xSz);

% crop output results array according to the actual iteration k, and print output result
objs = objs(1:(k+1));
regFunVals = regFunVals(1:(k+1));
duals = duals(1:(k+1));
ts = ts(:, 1:(k+1));
itersCG = itersCG(1:(k+1));
relResCG = relResCG(1:(k+1));
cpuTimes = cpuTimes(1:(k+1));
SNRs = SNRs(1:(k+1)); if isempty(xGT); SNRs = []; end
solver.printAlgorithmEnd('algorithmName', mfilename, 'cpuTime', cpuTimes(end), 'verbose', verbose, 'xGT', xGT, 'objFunValGT', objFunValGT, 'regWt', regWt, 'regFun', regFun);
end

function  [x0, s0]  = calcInitial(A, b, regWt)
% TODO:  epsilon: makes sure xmin/xmax >= epsilon,  smin/smax >= epsilon
epsilon = 1e-3; % a positive number much smaller than 1
% function handle to add a constant to z to make it z >=0;
getNonNegative = @(z) z + max(0, -min(z(:)));
% function handle to add a small constant to a z (already z>=0) so that zmin/zmax >= epsilon. Special case when z <=eps,
% increase every number by epsilon
getBalanced = @(z) z + max([0; (epsilon*max(z(:)) - min(z(:))) / (1-epsilon); double(max(z(:))<=eps)*epsilon]); 

% We known there is anlytical solution for inv(A*A') from the journal paper, but currently just do something simple.
isDebug = 1;
if isDebug
    x1 = epsilon*ones(size(A'*b));
else
    x1 = A'*((A*A')\b);
end


% increase x1 to make x2 nonegative: TODO: can we/should we make it strictly positive?
%x2 = x1 + max(0, -1.5*min(x1)); %
x2 = getNonNegative(x1); %x2 = x1 + max(0, -min(x1)); %
x2 = getBalanced(x2);

% increase x2 to x0, to make objFunGrad(x0) >= 0, and also increase again to make sure smin/smax >= epsilon (which is hard)
gradx2 = A'*(A*x2) - A'*b + regWt;
if all(gradx2 > 0) &&  (min(gradx2)/max(gradx2) > epsilon)
    x0 = x2;
    s0 = gradx2;
else
    ATAe = A'*(A*ones(size(x2)));
    if any(ATAe < 0)
        error('ATAe negative!');
    end
    delta2 = max(-gradx2 ./ ATAe); %delta2 = 1.5*max(-gradx2 ./ ATAe);
    x0 = x2 + delta2;
    s0 = gradx2 + delta2*ATAe;
end

s0 = getBalanced(s0);

if min(x0.*s0) < 0.0001*(x0'*s0/numel(x0))
    warning('x0.*s0 not balanced, the smallest one is too small');
end
    

% compute the initial point according to: 2006_Nocedal_Numerical Optimization, not good as it has large r_d = s0 - grad(x0)
%{
x1 = A'*((A*A')\b);
s1 = A'*(A*x1) - A'*b + regWt; % grad(h(x));

x2 = x1 + max(-1.5*min(x1), 0); % x1 + delta_x1
s2 = s1 + max(-1.5*min(s1), 0); % s1 + delta_s1

x2Ts = x2'*s2;
x0 = x2 + 0.5*x2Ts/sum(s2); % x2 + delta_x2
s0 = s2 + 0.5*x2Ts/sum(x2); % s2 + delta_s2;
%}
verbose = 1;
if verbose
    if ~exist('s2', 'var'); s2 = s0; end; if ~exist('s1', 'var'); s1 = s0; end
    figure, subplot(221); plot([x1 x2 x0]); legend('x1', 'x2', 'x0'); title('x1 -> x2 -> x0');
    subplot(222); plot([s1 s2 s0]); legend('s1', 's2', 's0'); title('s1 -> s2 -> s0');    
    r_dFun = @(x, s) s - (A'*(A*x) - A'*b + regWt);
    subplot(223); plot([r_dFun(x1,s1), r_dFun(x2,s2), r_dFun(x0,s0)]); legend('r_d1', 'r_d2', 'r_d0'); title('r_d');
    subplot(224); plot([x1.*s1, x2.*s2, x0.*s0]); legend('x1.*s1', 'x2.*s2', 'x0.*s0'); title('x.*s');
end
end
    
function  [dx, ds, relResCG_curIter, itersCG_curIter]  = calcNewtonStep(A, x, s, sigma, objFunGrad, tolCG, maxIterCG, MInvFun)
% Solve Newton step (dx, ds) from (x, s, sigma), where 
% A'*A*dx  - ds = r_d
% S*dx   + X*ds = r_c  ==> DInv*dx -  ds = r_c./x, where DInv = diag(1./d) and d = x./s;    
% where 
%  r_d = s - objFunGrad = s - (ATAx + regWt_ATb);
%  r_c = sigma*(x'*s/N) - x.*s;
% Inputs:
%   A, x, s, sigma, objFunGrad:  parameters for solve Newton Equation
%   tolCG, maxIterCG, MInvFun:  parameters to use PCG to solve dx from  H*dx = c

% 1 Solve H*dx = c
% H = (A'*A + diag(dInv)), where dInv = s./x = 1./d, and d = x./s
% c = r_d + r_c./x, where r_d = s - (ATAx + regWt_ATb) = 0 for feasible method, r_c = sigma*(x'*s/N) - x.*s;
N = numel(x);
d = x./s;
dInv = 1./d;
r_d = s - objFunGrad;
r_c = sigma*(x'*s/N) - x.*s;
c = r_d + r_c./x;
if exist('isADAT', 'var') && isADAT
    % woodbury inv(A'A+Dinv) = D-DA'inv(I+ADA')AD, it is good for small case, but extermely bad for large case?
    % dx = (D-DA'inv(I+ADA')AD)*c = Dc - DA'*inv(I+ADA')*ADc = Dc - DA'*dy
    % solve (I + A*D*A')*dy = ADc.  NOTE: the diag(I+ADA') is tricky to compute, so no diagonal preconditionning here
    ADATIFun = @(dy) dy + A*(d .* (A'*dy));
    c_y = A*(d.*c);
    dy0 = zeros(size(c_y));
    [dy, flag, relResCG_curIter, itersCG_curIter] = pcgXH(ADATIFun, c_y,optsIPPD.tolpcg, optsIPPD.mxiterpcg, [], [], dy0);
    dx = d.*(c - A'*dy);
else
    HFun = @(dx) A'*(A*dx) + dInv.*dx; % function handle of (Dinv + A'A)*dx
    dx0 = zeros(size(x));
    [dx, flag, relResCG_curIter, itersCG_curIter] = pcgXH(HFun, c, tolCG, maxIterCG, MInvFun, [], dx0); % %pcg(H,c,tol,maxit,M1,M2,x0); %matlab's own function
end

% Debug: generate full matrix for hessian and directly solve, this 
isDebugWithFullAMatrix = 0;
if exist('isDebugWithFullAMatrix', 'var') && isDebugWithFullAMatrix && ~isempty(opts.AMatrix)
    ATA = opts.AMatrix.'*opts.AMatrix;
    dxGT = (ATA + diag(1./d)) \ c;
    fprintf('The inexact Newton method has relative error in 2-norm: %.2e.\n', norm(dx-dxGT)/norm(dxGT));
    dx = dxGT;
end

% 2 solve ds
% todo: get A*dx from PCG results of dr = c - H*dx?  so no need to compute A*dx again
%ds = A'*(A*dx) - r_d; % feasible IPM , r_d will be zero if full step size was used
ds = r_c./x - dx./d; % Gondzio used, this is good to generate a balanced x*ds, though ds = A'*(A*dx) is better

% TODO: correct the error for ds?
end


function [dual, nu] = calcDual(r, objFunGrad, b, regWt)
if all(objFunGrad(:) >= 0)
    nu = -r;
else
    nu = -r * (-regWt/min(reshape(objFunGrad - regWt, [], 1))); % objFunGrad - regWt*ones(xSz) = A'(Ax-b) %2017.09.11_added 0.999
end
dual = - (0.5* nu(:) + b(:))'*nu(:);
end
