function f2 = squeezeWithBoundingBox(f, yxzMinMax, isPadding)
%function f2 = squeezeWithBoundingBox(f, yxzMinMax, isPadding)
%function f2 = squeezeWithBoundingBox(f, calcBoundingBox(f), isPadding)
%Shrink a volumetric image by removing its zero voxels outside the smallest bounding box
% Inputs:
%   f:      3D volumetric image represented by a 3D array,
%           or a 1D/2D cell array of 3D volumetric images
%   yxzMinMax: 2x3 matrix stores [yMin, xMin, zMin; yMax, xMax, zMax]
%   isPadding: default false.
% Outputs:
%   f2: the shrinked volumetric image
%           or a 1D/2D cell array of 3D volumetric images, where we will show 2D grid of figures
% See also:
%  calcBoundingBox.m

if ~exist('isPadding', 'var') || isempty(isPadding)
    isPadding = false; % change to false;
end

if ~exist('yxzMinMax', 'var') || isempty(yxzMinMax)
    yxzMinMax = calcBoundingBox(f);
end

% need to put cell case after handle parameters isPadding and yxzMinMax
if iscell(f)
    f2 = cell(size(f));
    for n = 1:numel(f)
        f2{n} = squeezeWithBoundingBox(f{n}, yxzMinMax, isPadding);
    end
    return
end



% Squeeze
f2 = f(yxzMinMax(1,1):yxzMinMax(2,1), yxzMinMax(1,2):yxzMinMax(2,2), yxzMinMax(1,3):yxzMinMax(2,3));

%% 2016.11.13_add optimzied spacing for bacteria double lope under default 3D view
% toto in future, adjustable to different 3D view and different content
if isPadding
    f2 = padVolume(f2);
%     spacing = max(ceil((yxzMinMax(2,:) - yxzMinMax(1,:)).*[0.5, 0.4, 0.3]*2), [1,1,1]);
%     %fSmall = padarray(fSmall, spacing, 0, 'both');
%     f2 = padarray(f2, spacing.*[1, 0, 0], 0, 'pre');
%     f2 = padarray(f2, spacing.*[0, 1, 0], 0, 'post');
%     f2 = padarray(f2, spacing.*[0, 0, 1], 0, 'pre');    
end
end

%% 2016.08.31_add spacing of 10% on x/y/z, after the padding, the center is still at (yxzMin+yxzMax)/2 of original
%spacing = max(ceil((yxzMinMax(2,:) - yxzMinMax(1,:))*0.1), [1,1,1]);
%fSmall = padarray(fSmall, spacing, 0, 'both');
